@@ -177,9 +177,7 @@ template <class T> class Mmap {
     // We don't check the return value because it fails in some environments.
     // For linux, in the kernel version >= 2.6.9, user process can mlock.
     // In older kernel, it fails if the process is running in user priviledge.
-#ifndef OS_WINDOWS
     mlock(p, length_);
-#endif
     text_ = reinterpret_cast<T *>(p);
 #else
     text_ = new T[length_];
@@ -202,12 +200,9 @@ template <class T> class Mmap {
 
     if (text_ != NULL) {
 #ifdef HAVE_MMAP
-#ifndef OS_WINDOWS
       // Again, we don't check the return value here.
       munlock(text_, length_);
-#endif  // OS_WINDOWS
       munmap(reinterpret_cast<char *>(text_), length_);
-      text_ = NULL;
 #else  // HAVE_MMAP
       if (flag_ == O_RDWR) {
         int fd2;
