@@ -29,12 +29,6 @@
 
 #include "base/scheduler.h"
 
-#ifdef OS_WINDOWS
-#include <time.h>  // time()
-#else
-#include <sys/time.h>  // time()
-#endif
-
 #include <cstdlib>
 #include <map>
 #include <utility>
@@ -140,7 +134,7 @@ class Job {
 class SchedulerImpl : public Scheduler::SchedulerInterface {
  public:
   SchedulerImpl() {
-    srand(static_cast<uint32>(time(NULL)));
+    Util::SetRandomSeed(static_cast<uint32>(Util::GetTime()));
   }
 
   virtual ~SchedulerImpl() {
@@ -241,7 +235,7 @@ class SchedulerImpl : public Scheduler::SchedulerInterface {
   uint32 CalcDelay(const Scheduler::JobSetting &job_setting) {
     uint32 delay = job_setting.delay_start();
     if (job_setting.random_delay() != 0) {
-      const uint64 r = rand();
+      const uint64 r = Util::Random(RAND_MAX);
       const uint64 d = job_setting.random_delay() * r;
       const uint64 random_delay = d / RAND_MAX;
       delay += random_delay;
