@@ -38,40 +38,54 @@
 #endif
 
 
+// Definition of TLS (Thread Local Storage) keyword.
 #ifndef TLS_KEYWORD
 
-// Definition of TLS (Thread Local Storage) keyword.
+#if defined(OS_WINDOWS) && defined(_MSC_VER)
 // Use VC's keyword on Windows.
-
+//
 // On Windows XP, thread local has a limitation. Since thread local
 // is now used inside converter and converter is a stand alone program,
 // it's not a issue at this moment.
 //
-// http://msdn.microsoft.com/en-us/library/9w1sdazb%28VS.80%29.aspx
+// http://msdn.microsoft.com/en-us/library/9w1sdazb.aspx
 // Thread-local variables work for EXEs and DLLs that are statically
 // linked to the main executable (directly or via another DLL) -
 // i.e. the DLL has to load before the main executable code starts
-//  running.
-#ifdef OS_WINDOWS
+// running.
 #define TLS_KEYWORD __declspec(thread)
 #define HAVE_TLS 1
-#endif // OS_WINDOWS
+#endif // OS_WINDOWS && _MSC_VER
 
-// Use GCC's keyword on Linux.
-#ifdef OS_LINUX
+// Andorid NDK doesn't support TLS.
+#if defined(OS_LINUX) && !defined(OS_ANDROID) && \
+    (defined(__GNUC__) || defined(__clang__))
+// GCC and Clang support TLS.
 #define TLS_KEYWORD __thread
 #define HAVE_TLS 1
-#endif  // OS_LINUX
+#endif  // OS_LINUX && !OS_ANDROID && (__GNUC__ || __clang__)
 
 
-// OSX doesn't support Thread Local Storage.
-#ifdef OS_MACOSX
+#if defined(OS_MACOSX) && \
+    defined(__GNUC__) && (__GNUC__ * 100 + __GNUC_MINOR__ >= 405)
+// GCC 4.5 and later can *emulate* TLS on Mac even though it is
+// expensive operation.
+#define TLS_KEYWORD __thread
+#define HAVE_TLS 1
+#endif  // OS_MACOSX && GCC 4.5 and later
+
+#endif  // !TLS_KEYWORD
+
+#ifndef TLS_KEYWORD
+// If TLS is not available, define TLS_KEYWORD as a dummy keyword.
 #define TLS_KEYWORD
-#undef HAVE_TLS
-#endif  // OS_MACOSX
+#endif  // !TLS_KEYWORD
 
+// Hereafter, we can use TLS_KEYWORD like a keyword.
+#ifndef TLS_KEYWORD
+#error TLS_KEYWORD should be defined here.
+#endif  // !TLS_KEYWORD
 
-#endif  // TLS_KEYWORD
 
 namespace mozc {
 
