@@ -30,6 +30,8 @@
 #include "composer/internal/composition.h"
 
 #include <algorithm>
+#include <set>
+#include <string>
 
 #include "composer/internal/char_chunk.h"
 #include "composer/internal/composition_input.h"
@@ -96,8 +98,7 @@ class CompositionTest : public testing::Test {
  protected:
   virtual void SetUp() {
     table_.reset(new Table);
-    composition_.reset(new Composition);
-    composition_->SetTable(table_.get());
+    composition_.reset(new Composition(table_.get()));
     composition_->SetInputMode(kConvT12r);
   }
 
@@ -168,8 +169,7 @@ TEST_F(CompositionTest, GetChunkLength) {
     // "っ"
     { "\xe3\x81\xa3", "ty", "tty", 3, 3 },
   };
-  Composition comp;
-  CharChunk *chunk = AppendChunk("", "", "", &comp);
+  CharChunk *chunk = AppendChunk("", "", "", composition_.get());
 
   for (int i = 0; i < ARRAYSIZE(test_cases); ++i) {
     const TestCase& test = test_cases[i];
@@ -214,79 +214,76 @@ bool TestGetChunkAt(Composition *comp,
 }  // anonymous namespace
 
 TEST_F(CompositionTest, GetChunkAt) {
-  Composition comp;
-  InitComposition(&comp);
+  InitComposition(composition_.get());
   const TransliteratorInterface *transliterator;
 
   transliterator = Transliterators::GetConversionStringSelector();
-  TestGetChunkAt(&comp, transliterator, 0, 0, 0);
-  TestGetChunkAt(&comp, transliterator, 1, 0, 1);
-  TestGetChunkAt(&comp, transliterator, 2, 1, 1);
-  TestGetChunkAt(&comp, transliterator, 3, 1, 2);
-  TestGetChunkAt(&comp, transliterator, 4, 2, 1);
-  TestGetChunkAt(&comp, transliterator, 5, 3, 1);
-  TestGetChunkAt(&comp, transliterator, 6, 3, 2);
-  TestGetChunkAt(&comp, transliterator, 7, 4, 1);
-  TestGetChunkAt(&comp, transliterator, 8, 4, 2);
-  TestGetChunkAt(&comp, transliterator, 9, 4, 3);
-  TestGetChunkAt(&comp, transliterator, 10, 4, 3);  // end
-  TestGetChunkAt(&comp, transliterator, 11, 4, 3);  // end
+  TestGetChunkAt(composition_.get(), transliterator, 0, 0, 0);
+  TestGetChunkAt(composition_.get(), transliterator, 1, 0, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 2, 1, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 3, 1, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 4, 2, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 5, 3, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 6, 3, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 7, 4, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 8, 4, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 9, 4, 3);
+  TestGetChunkAt(composition_.get(), transliterator, 10, 4, 3);  // end
+  TestGetChunkAt(composition_.get(), transliterator, 11, 4, 3);  // end
 
   transliterator = Transliterators::GetRawStringSelector();
-  TestGetChunkAt(&comp, transliterator, 0, 0, 0);
-  TestGetChunkAt(&comp, transliterator, 1, 0, 1);
-  TestGetChunkAt(&comp, transliterator, 2, 1, 1);
-  TestGetChunkAt(&comp, transliterator, 3, 1, 2);
-  TestGetChunkAt(&comp, transliterator, 4, 2, 1);
-  TestGetChunkAt(&comp, transliterator, 5, 2, 2);
-  TestGetChunkAt(&comp, transliterator, 6, 3, 1);
-  TestGetChunkAt(&comp, transliterator, 7, 3, 2);
-  TestGetChunkAt(&comp, transliterator, 8, 3, 3);
-  TestGetChunkAt(&comp, transliterator, 9, 4, 1);
-  TestGetChunkAt(&comp, transliterator, 10, 4, 2);
-  TestGetChunkAt(&comp, transliterator, 11, 4, 3);
-  TestGetChunkAt(&comp, transliterator, 12, 4, 3);  // end
-  TestGetChunkAt(&comp, transliterator, 13, 4, 3);  // end
+  TestGetChunkAt(composition_.get(), transliterator, 0, 0, 0);
+  TestGetChunkAt(composition_.get(), transliterator, 1, 0, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 2, 1, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 3, 1, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 4, 2, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 5, 2, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 6, 3, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 7, 3, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 8, 3, 3);
+  TestGetChunkAt(composition_.get(), transliterator, 9, 4, 1);
+  TestGetChunkAt(composition_.get(), transliterator, 10, 4, 2);
+  TestGetChunkAt(composition_.get(), transliterator, 11, 4, 3);
+  TestGetChunkAt(composition_.get(), transliterator, 12, 4, 3);  // end
+  TestGetChunkAt(composition_.get(), transliterator, 13, 4, 3);  // end
 }
 
 TEST_F(CompositionTest, GetString) {
-  Composition comp;
-  InitComposition(&comp);
+  InitComposition(composition_.get());
   string composition;
 
   const size_t dummy_position = 0;
 
   // Test RAW mode
-  comp.SetDisplayMode(dummy_position,
+  composition_->SetDisplayMode(dummy_position,
                       Transliterators::GetRawStringSelector());
-  comp.GetString(&composition);
+  composition_->GetString(&composition);
   EXPECT_EQ("akykittatty", composition);
 
   // Test CONVERSION mode
-  comp.SetDisplayMode(dummy_position,
+  composition_->SetDisplayMode(dummy_position,
                       Transliterators::GetConversionStringSelector());
-  comp.GetString(&composition);
+  composition_->GetString(&composition);
   // "あkyきったっty"
   EXPECT_EQ("\xe3\x81\x82\x6b\x79\xe3\x81\x8d\xe3\x81\xa3\xe3\x81\x9f\xe3\x81"
             "\xa3\x74\x79", composition);
 }
 
 TEST_F(CompositionTest, GetStringWithDisplayMode) {
-  Composition comp;
   // "も"
-  AppendChunk("\xe3\x82\x82", "", "mo", &comp);
+  AppendChunk("\xe3\x82\x82", "", "mo", composition_.get());
   // "ず"
-  AppendChunk("\xe3\x81\x9a", "", "z", &comp);
+  AppendChunk("\xe3\x81\x9a", "", "z", composition_.get());
   // "く"
-  AppendChunk("\xe3\x81\x8f", "", "c", &comp);
+  AppendChunk("\xe3\x81\x8f", "", "c", composition_.get());
 
   string composition;
-  comp.GetStringWithTransliterator(
+  composition_->GetStringWithTransliterator(
       Transliterators::GetConversionStringSelector(), &composition);
   // "もずく"
   EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a\xe3\x81\x8f", composition);
 
-  comp.GetStringWithTransliterator(
+  composition_->GetStringWithTransliterator(
       Transliterators::GetRawStringSelector(), &composition);
   EXPECT_EQ("mozc", composition);
 }
@@ -427,37 +424,44 @@ TEST_F(CompositionTest, MaybeSplitChunkAt) {
   for (int i = 0; i < ARRAYSIZE(test_cases); ++i) {
     const TestCase& test = test_cases[i];
 
-    // Test RAW mode
-    Composition raw_comp;
-    InitComposition(&raw_comp);
-    raw_comp.SetDisplayMode(dummy_position, kRawT12r);
-    CharChunkList::iterator raw_it;
+    {  // Test RAW mode
+      Composition raw_comp(table_.get());
+      InitComposition(&raw_comp);
+      raw_comp.SetDisplayMode(dummy_position, kRawT12r);
+      CharChunkList::iterator raw_it;
 
-    raw_comp.MaybeSplitChunkAt(test.position, &raw_it);
-    const size_t raw_chunks_size = raw_comp.GetCharChunkList().size();
-    EXPECT_EQ(test.expected_raw_chunks_size, raw_chunks_size);
+      raw_comp.MaybeSplitChunkAt(test.position, &raw_it);
+      const size_t raw_chunks_size = raw_comp.GetCharChunkList().size();
+      EXPECT_EQ(test.expected_raw_chunks_size, raw_chunks_size);
+    }
 
-    // Test CONVERSION mode
-    Composition conv_comp;
-    InitComposition(&conv_comp);
-    conv_comp.SetDisplayMode(dummy_position, kConvT12r);
-    CharChunkList::iterator conv_it;
+    {  // Test CONVERSION mode
+      Composition conv_comp(table_.get());
+      InitComposition(&conv_comp);
+      conv_comp.SetDisplayMode(dummy_position, kConvT12r);
+      CharChunkList::iterator conv_it;
 
-    conv_comp.MaybeSplitChunkAt(test.position, &conv_it);
-    const size_t conv_chunks_size = conv_comp.GetCharChunkList().size();
-    EXPECT_EQ(test.expected_conv_chunks_size, conv_chunks_size);
+      conv_comp.MaybeSplitChunkAt(test.position, &conv_it);
+      const size_t conv_chunks_size = conv_comp.GetCharChunkList().size();
+      EXPECT_EQ(test.expected_conv_chunks_size, conv_chunks_size);
+    }
   }
 }
 
 namespace {
 string GetDeletedString(const TransliteratorInterface* t12r,
                         const int position) {
-  Composition comp;
-  InitComposition(&comp);
-  comp.SetDisplayMode(0, t12r);
-  comp.DeleteAt(position);
+  scoped_ptr<Table> table(new Table);
+  scoped_ptr<Composition> comp(new Composition(table.get()));
+
+  InitComposition(comp.get());
+  comp->SetDisplayMode(0, t12r);
+  comp->DeleteAt(position);
   string composition;
-  comp.GetString(&composition);
+  comp->GetString(&composition);
+
+  comp.reset();
+  table.reset();
   return composition;
 }
 }  // anonymous namespace
@@ -522,21 +526,20 @@ TEST_F(CompositionTest, DeleteAt) {
 }
 
 TEST_F(CompositionTest, DeleteAt_InvisibleCharacter) {
-  Composition comp;
   CharChunkList::iterator it;
   CharChunk *chunk;
 
-  comp.MaybeSplitChunkAt(0, &it);
+  composition_->MaybeSplitChunkAt(0, &it);
 
-  chunk = *comp.InsertChunk(&it);
+  chunk = *composition_->InsertChunk(&it);
   chunk->set_raw("1");
   chunk->set_pending(Table::ParseSpecialKey("{1}"));
 
-  chunk = *comp.InsertChunk(&it);
+  chunk = *composition_->InsertChunk(&it);
   chunk->set_raw("2");
   chunk->set_pending(Table::ParseSpecialKey("{2}2"));
 
-  chunk = *comp.InsertChunk(&it);
+  chunk = *composition_->InsertChunk(&it);
   chunk->set_raw("3");
   chunk->set_pending("3");
 
@@ -544,9 +547,9 @@ TEST_F(CompositionTest, DeleteAt_InvisibleCharacter) {
   // (raw, pending) = [ ("1", "{1}"), ("2", "{2}2"), ("3", "3") ]
   // {} means invisible characters.
 
-  comp.DeleteAt(0);
+  composition_->DeleteAt(0);
   string composition;
-  comp.GetString(&composition);
+  composition_->GetString(&composition);
   EXPECT_EQ("3", composition);
 }
 
@@ -574,17 +577,21 @@ void InitTable(Table* table) {
 string GetInsertedString(const TransliteratorInterface* t12r,
                          const size_t position,
                          const string &input) {
-  Composition comp;
-  InitComposition(&comp);
+  scoped_ptr<Table> table(new Table);
+  InitTable(table.get());
+  scoped_ptr<Composition> comp(new Composition(table.get()));
+  InitComposition(comp.get());
 
-  Table table;
-  InitTable(&table);
-  comp.SetTable(&table);
-  comp.SetDisplayMode(0, t12r);
-  comp.InsertAt(position, input);
+  comp->SetTableForUnittest(table.get());
+  comp->SetDisplayMode(0, t12r);
+  comp->InsertAt(position, input);
 
   string composition;
-  comp.GetString(&composition);
+  comp->GetString(&composition);
+
+  comp.reset();
+  table.reset();
+
   return composition;
 }
 }  // anonymous namespace
@@ -685,17 +692,16 @@ TEST_F(CompositionTest, InsertAt) {
 }
 
 TEST_F(CompositionTest, GetExpandedStrings) {
-  Composition comp;
-  InitComposition(&comp);
+  InitComposition(composition_.get());
 
   Table table;
   InitTable(&table);
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   // a ky ki tta tty
   string base;
   set<string> expanded;
-  comp.GetExpandedStrings(&base, &expanded);
+  composition_->GetExpandedStrings(&base, &expanded);
   // "あkyきったっ"
   EXPECT_EQ(
       "\xe3\x81\x82\x6b\x79\xe3\x81\x8d\xe3\x81\xa3\xe3\x81\x9f\xe3\x81\xa3",
@@ -713,93 +719,90 @@ TEST_F(CompositionTest, GetExpandedStrings) {
 
 TEST_F(CompositionTest, ConvertPosition) {
   // Test against http://b/1550597
-  Composition comp;
 
   // Invalid positions.
-  EXPECT_EQ(0, comp.ConvertPosition(static_cast<size_t>(-1),
+  EXPECT_EQ(0, composition_->ConvertPosition(static_cast<size_t>(-1),
                                     kConvT12r, kRawT12r));
-  EXPECT_EQ(0, comp.ConvertPosition(0, kConvT12r, kRawT12r));
-  EXPECT_EQ(0, comp.ConvertPosition(1, kConvT12r, kRawT12r));
-  EXPECT_EQ(0, comp.ConvertPosition(0, kRawT12r, kConvT12r));
-  EXPECT_EQ(0, comp.ConvertPosition(static_cast<size_t>(-1),
+  EXPECT_EQ(0, composition_->ConvertPosition(0, kConvT12r, kRawT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(1, kConvT12r, kRawT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(0, kRawT12r, kConvT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(static_cast<size_t>(-1),
                                     kRawT12r, kConvT12r));
-  EXPECT_EQ(0, comp.ConvertPosition(1, kRawT12r, kConvT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(1, kRawT12r, kConvT12r));
 
   // "ね"
-  AppendChunk("\xe3\x81\xad", "", "ne", &comp);
+  AppendChunk("\xe3\x81\xad", "", "ne", composition_.get());
   // "っと"
-  AppendChunk("\xe3\x81\xa3\xe3\x81\xa8", "", "tto", &comp);
+  AppendChunk("\xe3\x81\xa3\xe3\x81\xa8", "", "tto", composition_.get());
 
   // "|ねっと" -> "|netto"
-  EXPECT_EQ(0, comp.ConvertPosition(0, kConvT12r, kRawT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(0, kConvT12r, kRawT12r));
   // "ね|っと" -> "ne|tto"
-  EXPECT_EQ(2, comp.ConvertPosition(1, kConvT12r, kRawT12r));
+  EXPECT_EQ(2, composition_->ConvertPosition(1, kConvT12r, kRawT12r));
   // "ねっ|と" -> "net|to"
-  EXPECT_EQ(3, comp.ConvertPosition(2, kConvT12r, kRawT12r));
+  EXPECT_EQ(3, composition_->ConvertPosition(2, kConvT12r, kRawT12r));
   // "ねっと|" -> "netto|"
-  EXPECT_EQ(5, comp.ConvertPosition(3, kConvT12r, kRawT12r));
+  EXPECT_EQ(5, composition_->ConvertPosition(3, kConvT12r, kRawT12r));
 
   // Invalid positions.
-  EXPECT_EQ(5, comp.ConvertPosition(static_cast<size_t>(-1),
+  EXPECT_EQ(5, composition_->ConvertPosition(static_cast<size_t>(-1),
                                     kConvT12r, kRawT12r));
-  EXPECT_EQ(5, comp.ConvertPosition(4, kConvT12r, kRawT12r));
+  EXPECT_EQ(5, composition_->ConvertPosition(4, kConvT12r, kRawT12r));
 
   // "|netto" -> "|ねっと"
-  EXPECT_EQ(0, comp.ConvertPosition(0, kRawT12r, kConvT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(0, kRawT12r, kConvT12r));
   // "n|etto" -> "ね|っと"
-  EXPECT_EQ(1, comp.ConvertPosition(1, kRawT12r, kConvT12r));
+  EXPECT_EQ(1, composition_->ConvertPosition(1, kRawT12r, kConvT12r));
   // "ne|tto" -> "ね|っと"
-  EXPECT_EQ(1, comp.ConvertPosition(2, kRawT12r, kConvT12r));
+  EXPECT_EQ(1, composition_->ConvertPosition(2, kRawT12r, kConvT12r));
   // "net|to" -> "ねっ|と"
-  EXPECT_EQ(2, comp.ConvertPosition(3, kRawT12r, kConvT12r));
+  EXPECT_EQ(2, composition_->ConvertPosition(3, kRawT12r, kConvT12r));
   // "nett|o" -> "ねっと|"
-  EXPECT_EQ(3, comp.ConvertPosition(4, kRawT12r, kConvT12r));
+  EXPECT_EQ(3, composition_->ConvertPosition(4, kRawT12r, kConvT12r));
   // "netto|" -> "ねっと|"
-  EXPECT_EQ(3, comp.ConvertPosition(5, kRawT12r, kConvT12r));
+  EXPECT_EQ(3, composition_->ConvertPosition(5, kRawT12r, kConvT12r));
   // Invalid positions.
-  EXPECT_EQ(3, comp.ConvertPosition(static_cast<size_t>(-1),
+  EXPECT_EQ(3, composition_->ConvertPosition(static_cast<size_t>(-1),
                                     kRawT12r, kConvT12r));
-  EXPECT_EQ(3, comp.ConvertPosition(6, kRawT12r, kConvT12r));
+  EXPECT_EQ(3, composition_->ConvertPosition(6, kRawT12r, kConvT12r));
 
   CharChunkList::iterator chunk_it;
   size_t inner_position;
-  comp.GetChunkAt(5, kRawT12r, &chunk_it, &inner_position);
+  composition_->GetChunkAt(5, kRawT12r, &chunk_it, &inner_position);
 
   EXPECT_EQ("tto", (*chunk_it)->raw());
   EXPECT_EQ(3, inner_position);
 }
 
 TEST_F(CompositionTest, SetDisplayMode) {
-  Composition comp;
   // "も"
-  AppendChunk("\xe3\x82\x82", "", "mo", &comp);
+  AppendChunk("\xe3\x82\x82", "", "mo", composition_.get());
   // "ず"
-  AppendChunk("\xe3\x81\x9a", "", "zu", &comp);
+  AppendChunk("\xe3\x81\x9a", "", "zu", composition_.get());
   // "く"
-  AppendChunk("\xe3\x81\x8f", "", "ku", &comp);
+  AppendChunk("\xe3\x81\x8f", "", "ku", composition_.get());
 
   CharChunkList::iterator chunk_it;
   size_t inner_position;
-  comp.GetChunkAt(0, kConvT12r, &chunk_it, &inner_position);
+  composition_->GetChunkAt(0, kConvT12r, &chunk_it, &inner_position);
   EXPECT_EQ("mo", (*chunk_it)->raw());
   EXPECT_EQ(0, inner_position);
-  comp.GetChunkAt(1, kConvT12r, &chunk_it, &inner_position);
+  composition_->GetChunkAt(1, kConvT12r, &chunk_it, &inner_position);
   EXPECT_EQ("mo", (*chunk_it)->raw());
   EXPECT_EQ(1, inner_position);
-  comp.GetChunkAt(2, kConvT12r, &chunk_it, &inner_position);
+  composition_->GetChunkAt(2, kConvT12r, &chunk_it, &inner_position);
   EXPECT_EQ("zu", (*chunk_it)->raw());
   EXPECT_EQ(1, inner_position);
-  comp.GetChunkAt(3, kConvT12r, &chunk_it, &inner_position);
+  composition_->GetChunkAt(3, kConvT12r, &chunk_it, &inner_position);
   EXPECT_EQ("ku", (*chunk_it)->raw());
   EXPECT_EQ(1, inner_position);
 
-  EXPECT_EQ(6, comp.SetDisplayMode(1, kRawT12r));
-  EXPECT_EQ(3, comp.SetDisplayMode(2, kConvT12r));
-  EXPECT_EQ(6, comp.SetDisplayMode(2, kRawT12r));
+  EXPECT_EQ(6, composition_->SetDisplayMode(1, kRawT12r));
+  EXPECT_EQ(3, composition_->SetDisplayMode(2, kConvT12r));
+  EXPECT_EQ(6, composition_->SetDisplayMode(2, kRawT12r));
 }
 
 TEST_F(CompositionTest, GetStringWithTrimMode) {
-  Composition comp;
   Table table;
   // "か"
   table.AddRule("ka", "\xe3\x81\x8b", "");
@@ -808,65 +811,63 @@ TEST_F(CompositionTest, GetStringWithTrimMode) {
   // This makes the above rule ambiguous.
   // "な"
   table.AddRule("na", "\xe3\x81\xaa", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   string output_empty;
-  comp.GetStringWithTrimMode(TRIM, &output_empty);
+  composition_->GetStringWithTrimMode(TRIM, &output_empty);
   EXPECT_TRUE(output_empty.empty());
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "k");
-  pos = comp.InsertAt(pos, "a");
-  pos = comp.InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "k");
+  pos = composition_->InsertAt(pos, "a");
+  pos = composition_->InsertAt(pos, "n");
 
   string output_trim;
-  comp.GetStringWithTrimMode(TRIM, &output_trim);
+  composition_->GetStringWithTrimMode(TRIM, &output_trim);
   // "か"
   EXPECT_EQ("\xe3\x81\x8b", output_trim);
 
   string output_asis;
-  comp.GetStringWithTrimMode(ASIS, &output_asis);
+  composition_->GetStringWithTrimMode(ASIS, &output_asis);
   // "かn"
   EXPECT_EQ("\xe3\x81\x8b\x6e", output_asis);
 
   string output_fix;
-  comp.GetStringWithTrimMode(FIX, &output_asis);
+  composition_->GetStringWithTrimMode(FIX, &output_asis);
   // "かん"
   EXPECT_EQ("\xe3\x81\x8b\xe3\x82\x93", output_asis);
 }
 
 TEST_F(CompositionTest, InsertKeyAndPreeditAt) {
-  Composition comp;
   Table table;
   // "す゛", "ず"
   table.AddRule("\xe3\x81\x99\xe3\x82\x9b", "\xe3\x81\x9a", "");
   // "く゛", "ぐ"
   table.AddRule("\xe3\x81\x8f\xe3\x82\x9b", "\xe3\x81\x90", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
   // "も"
-  pos = comp.InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
   // "す"
-  pos = comp.InsertKeyAndPreeditAt(pos, "r", "\xe3\x81\x99");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "r", "\xe3\x81\x99");
   // "゛"
-  pos = comp.InsertKeyAndPreeditAt(pos, "@", "\xe3\x82\x9b");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "@", "\xe3\x82\x9b");
   // "く"
-  pos = comp.InsertKeyAndPreeditAt(pos, "h", "\xe3\x81\x8f");
-  pos = comp.InsertKeyAndPreeditAt(pos, "!", "!");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "h", "\xe3\x81\x8f");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "!", "!");
 
   string comp_str;
-  comp.GetString(&comp_str);
+  composition_->GetString(&comp_str);
   // "もずく!"
   EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a\xe3\x81\x8f\x21", comp_str);
 
   string comp_ascii_str;
-  comp.GetStringWithTransliterator(kRawT12r, &comp_ascii_str);
+  composition_->GetStringWithTransliterator(kRawT12r, &comp_ascii_str);
   EXPECT_EQ("mr@h!", comp_ascii_str);
 }
 
 TEST_F(CompositionTest, InsertKey_ForN) {
-  Composition comp;
   Table table;
   table.AddRule("a", "[A]", "");
   table.AddRule("n", "[N]", "");
@@ -875,139 +876,135 @@ TEST_F(CompositionTest, InsertKey_ForN) {
   table.AddRule("nya", "[NYA]", "");
   table.AddRule("ya", "[YA]", "");
   table.AddRule("ka", "[KA]", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "n");
-  pos = comp.InsertAt(pos, "y");
-  pos = comp.InsertAt(pos, "n");
-  pos = comp.InsertAt(pos, "y");
-  pos = comp.InsertAt(pos, "a");
+  pos = composition_->InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "y");
+  pos = composition_->InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "y");
+  pos = composition_->InsertAt(pos, "a");
 
   string comp_str;
-  comp.GetString(&comp_str);
+  composition_->GetString(&comp_str);
   EXPECT_EQ("ny[NYA]", comp_str);
 }
 
 TEST_F(CompositionTest, GetStringWithDisplayMode_ForKana) {
-  Composition comp;
   Table table;
   // Empty table is OK.
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
   // "も"
-  pos = comp.InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
 
   string comp_str;
-  comp.GetStringWithTransliterator(kRawT12r, &comp_str);
+  composition_->GetStringWithTransliterator(kRawT12r, &comp_str);
   EXPECT_EQ("m", comp_str);
 }
 
 TEST_F(CompositionTest, InputMode) {
-  Composition comp;
   Table table;
   // "あ"
   table.AddRule("a", "\xE3\x81\x82", "");
   // "か"
   table.AddRule("ka", "\xE3\x81\x8B", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "k");
+  pos = composition_->InsertAt(pos, "k");
 
   string result;
-  comp.GetString(&result);
+  composition_->GetString(&result);
   // "k"
   EXPECT_EQ("k", result);
 
-  comp.SetInputMode(kFullKatakanaT12r);
-  pos = comp.InsertAt(pos, "a");
-  comp.GetString(&result);
+  composition_->SetInputMode(kFullKatakanaT12r);
+  pos = composition_->InsertAt(pos, "a");
+  composition_->GetString(&result);
   // If a vowel and a consonant were typed with different
   // transliterators, these characters should not be combined.
   // "kア"
   EXPECT_EQ("\x6B\xE3\x82\xA2", result);
 
-  comp.SetInputMode(kHalfAsciiT12r);
-  pos = comp.InsertAt(pos, "k");
-  comp.GetString(&result);
+  composition_->SetInputMode(kHalfAsciiT12r);
+  pos = composition_->InsertAt(pos, "k");
+  composition_->GetString(&result);
   // "kアk"
   EXPECT_EQ("\x6B\xE3\x82\xA2\x6B", result);
 
-  comp.SetInputMode(kHiraganaT12r);
-  pos = comp.InsertAt(pos, "a");
-  comp.GetString(&result);
+  composition_->SetInputMode(kHiraganaT12r);
+  pos = composition_->InsertAt(pos, "a");
+  composition_->GetString(&result);
   // "kアkあ"
   EXPECT_EQ("\x6B\xE3\x82\xA2\x6B\xE3\x81\x82", result);
 
   // "|kアkあ"
-  EXPECT_EQ(kDefaultT12r, comp.GetTransliterator(0));
+  EXPECT_EQ(kDefaultT12r, composition_->GetTransliterator(0));
   // "k|アkあ"
-  EXPECT_EQ(kDefaultT12r, comp.GetTransliterator(1));
+  EXPECT_EQ(kDefaultT12r, composition_->GetTransliterator(1));
   // "kア|kあ"
-  EXPECT_EQ(kFullKatakanaT12r, comp.GetTransliterator(2));
+  EXPECT_EQ(kFullKatakanaT12r, composition_->GetTransliterator(2));
   // "kアk|あ"
-  EXPECT_EQ(kHalfAsciiT12r, comp.GetTransliterator(3));
+  EXPECT_EQ(kHalfAsciiT12r, composition_->GetTransliterator(3));
   // "kアkあ|"
-  EXPECT_EQ(kHiraganaT12r, comp.GetTransliterator(4));
+  EXPECT_EQ(kHiraganaT12r, composition_->GetTransliterator(4));
   // "kアkあ...|"
-  EXPECT_EQ(kHiraganaT12r, comp.GetTransliterator(5));
-  EXPECT_EQ(kHiraganaT12r, comp.GetTransliterator(10));
+  EXPECT_EQ(kHiraganaT12r, composition_->GetTransliterator(5));
+  EXPECT_EQ(kHiraganaT12r, composition_->GetTransliterator(10));
 }
 
 TEST_F(CompositionTest, Transliterator) {
-  Composition comp;
   Table table;
   // "あ"
   table.AddRule("a", "\xE3\x81\x82", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   // Insert "a" which is converted to "あ".
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "a");
+  pos = composition_->InsertAt(pos, "a");
   EXPECT_EQ(1, pos);
   string result;
-  comp.GetString(&result);
+  composition_->GetString(&result);
   // "あ"
   EXPECT_EQ("\xE3\x81\x82", result);
 
   // Set transliterator for Half Ascii.
-  comp.SetTransliterator(0, pos, kHalfAsciiT12r);
-  comp.GetString(&result);
+  composition_->SetTransliterator(0, pos, kHalfAsciiT12r);
+  composition_->GetString(&result);
   EXPECT_EQ("a", result);
 
   // Insert "a" again.
-  pos = comp.InsertAt(pos, "a");
+  pos = composition_->InsertAt(pos, "a");
   EXPECT_EQ(2, pos);
   result.clear();
-  comp.GetString(&result);
+  composition_->GetString(&result);
   // "aあ"
   EXPECT_EQ("a\xE3\x81\x82", result);
 
   // Set transliterator for Full Katakana.
-  comp.SetTransliterator(0, pos, kFullKatakanaT12r);
-  comp.GetString(&result);
+  composition_->SetTransliterator(0, pos, kFullKatakanaT12r);
+  composition_->GetString(&result);
   // "アア"
   EXPECT_EQ("\xE3\x82\xA2\xE3\x82\xA2", result);
 }
 
 TEST_F(CompositionTest, HalfAsciiTransliterator) {
-  Composition comp;
   Table table;
   // "ー"
   table.AddRule("-", "\xE3\x83\xBC", "");
-  comp.SetTable(&table);
-  comp.SetInputMode(kHalfAsciiT12r);
+  composition_->SetTableForUnittest(&table);
+  composition_->SetInputMode(kHalfAsciiT12r);
 
   size_t pos = 0;
-  pos = comp.InsertKeyAndPreeditAt(pos, "-", "-");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "-", "-");
   EXPECT_EQ(1, pos);
-  EXPECT_EQ("-", GetString(comp));
+  EXPECT_EQ("-", GetString(*composition_));
 
-  pos = comp.InsertKeyAndPreeditAt(pos, "-", "-");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "-", "-");
   EXPECT_EQ(2, pos);
-  EXPECT_EQ("--", GetString(comp));
+  EXPECT_EQ("--", GetString(*composition_));
 }
 
 TEST_F(CompositionTest, ShouldCommit) {
@@ -1049,64 +1046,62 @@ TEST_F(CompositionTest, ShouldCommit) {
 
 TEST_F(CompositionTest, Issue2190364) {
   // This is a unittest against http://b/2190364
-  Composition comp;
   Table table;
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
 
-  comp.SetInputMode(kFullAsciiT12r);
+  composition_->SetInputMode(kFullAsciiT12r);
   // "ち"
-  pos = comp.InsertKeyAndPreeditAt(pos, "a", "\xE3\x81\xA1");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "a", "\xE3\x81\xA1");
   // "ａ"
-  EXPECT_EQ("\xEF\xBD\x81", GetString(comp));
+  EXPECT_EQ("\xEF\xBD\x81", GetString(*composition_));
 
-  pos = comp.InsertAt(pos, " ");
+  pos = composition_->InsertAt(pos, " ");
   // "ａ　"
-  EXPECT_EQ("\xEF\xBD\x81\xE3\x80\x80", GetString(comp));
+  EXPECT_EQ("\xEF\xBD\x81\xE3\x80\x80", GetString(*composition_));
 }
 
 
 TEST_F(CompositionTest, Issue1817410) {
   // This is a unittest against http://b/2190364
-  Composition comp;
   Table table;
   // "っ"
   table.AddRule("ss", "\xE3\x81\xA3", "s");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "s");
-  pos = comp.InsertAt(pos, "s");
+  pos = composition_->InsertAt(pos, "s");
+  pos = composition_->InsertAt(pos, "s");
 
   string preedit;
-  comp.GetString(&preedit);
+  composition_->GetString(&preedit);
   // "っs"
   EXPECT_EQ("\xE3\x81\xA3\x73", preedit);
 
-  EXPECT_EQ(0, comp.ConvertPosition(0, kNullT12r, kHalfAsciiT12r));
-  EXPECT_EQ(1, comp.ConvertPosition(1, kNullT12r, kHalfAsciiT12r));
-  EXPECT_EQ(2, comp.ConvertPosition(2, kNullT12r, kHalfAsciiT12r));
+  EXPECT_EQ(0, composition_->ConvertPosition(0, kNullT12r, kHalfAsciiT12r));
+  EXPECT_EQ(1, composition_->ConvertPosition(1, kNullT12r, kHalfAsciiT12r));
+  EXPECT_EQ(2, composition_->ConvertPosition(2, kNullT12r, kHalfAsciiT12r));
 
   {  // "s|s"
     CharChunkList::iterator chunk_it;
     size_t inner_position;
-    comp.GetChunkAt(1, kNullT12r, &chunk_it, &inner_position);
+    composition_->GetChunkAt(1, kNullT12r, &chunk_it, &inner_position);
     EXPECT_EQ(1, inner_position);
     EXPECT_EQ(2, (*chunk_it)->GetLength(kNullT12r));
 
-    EXPECT_EQ(0, comp.GetPosition(kHalfAsciiT12r, chunk_it));
+    EXPECT_EQ(0, composition_->GetPosition(kHalfAsciiT12r, chunk_it));
     EXPECT_EQ(2, (*chunk_it)->GetLength(kHalfAsciiT12r));
   }
 
   {  // "ss|"
     CharChunkList::iterator chunk_it;
     size_t inner_position;
-    comp.GetChunkAt(2, kNullT12r, &chunk_it, &inner_position);
+    composition_->GetChunkAt(2, kNullT12r, &chunk_it, &inner_position);
     EXPECT_EQ(2, inner_position);
     EXPECT_EQ(2, (*chunk_it)->GetLength(kNullT12r));
 
-    EXPECT_EQ(0, comp.GetPosition(kHalfAsciiT12r, chunk_it));
+    EXPECT_EQ(0, composition_->GetPosition(kHalfAsciiT12r, chunk_it));
     EXPECT_EQ(2, (*chunk_it)->GetLength(kHalfAsciiT12r));
   }
 }
@@ -1114,29 +1109,27 @@ TEST_F(CompositionTest, Issue1817410) {
 TEST_F(CompositionTest, Issue2209634) {
   // This is a unittest against http://b/2209634
   // "q@" becomes "qた@".
-  Composition comp;
   Table table;
   // "た"
   table.AddRule("q", "", "\xE3\x81\x9F");
   // "た@", "だ"
   table.AddRule("\xE3\x81\x9F\x40", "\xE3\x81\xA0", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
-  comp.SetInputMode(kHalfAsciiT12r);
+  composition_->SetInputMode(kHalfAsciiT12r);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "q");
-  pos = comp.InsertAt(pos, "@");
+  pos = composition_->InsertAt(pos, "q");
+  pos = composition_->InsertAt(pos, "@");
 
   string preedit;
-  comp.GetString(&preedit);
+  composition_->GetString(&preedit);
   EXPECT_EQ("q@", preedit);
 }
 
 TEST_F(CompositionTest, Issue2330530) {
   // This is a unittest against http://b/2330530
   // "Win" + Numpad7 becomes "Win77" instead of "Win7".
-  Composition comp;
   Table table;
   // "うぃ"
   table.AddRule("wi", "\xe3\x81\x86\xe3\x81\x83", "");
@@ -1146,21 +1139,21 @@ TEST_F(CompositionTest, Issue2330530) {
   table.AddRule("n", "\xe3\x82\x93", "");
   // "な"
   table.AddRule("na", "\xe3\x81\xaa", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
-  comp.SetInputMode(kHalfAsciiT12r);
+  composition_->SetInputMode(kHalfAsciiT12r);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "W");
-  pos = comp.InsertAt(pos, "i");
-  pos = comp.InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "W");
+  pos = composition_->InsertAt(pos, "i");
+  pos = composition_->InsertAt(pos, "n");
 
   string preedit;
-  comp.GetString(&preedit);
+  composition_->GetString(&preedit);
   EXPECT_EQ("Win", preedit);
 
-  pos = comp.InsertKeyAndPreeditAt(pos, "7", "7");
-  comp.GetString(&preedit);
+  pos = composition_->InsertKeyAndPreeditAt(pos, "7", "7");
+  composition_->GetString(&preedit);
   EXPECT_EQ("Win7", preedit);
 }
 
@@ -1181,25 +1174,24 @@ TEST_F(CompositionTest, Issue2819580) {
   // "びょ"
   table.AddRule("byo", "\xe3\x81\xb3\xe3\x82\x87", "");
 
-  Composition comp;
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
-  comp.SetInputMode(kHiraganaT12r);
+  composition_->SetInputMode(kHiraganaT12r);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "n");
-  pos = comp.InsertAt(pos, "y");
+  pos = composition_->InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "y");
   {
     string output;
-    comp.GetStringWithTrimMode(FIX, &output);
+    composition_->GetStringWithTrimMode(FIX, &output);
     // "んｙ"
     EXPECT_EQ("\xe3\x82\x93\xef\xbd\x99", output);
 
-    comp.GetStringWithTrimMode(ASIS, &output);
+    composition_->GetStringWithTrimMode(ASIS, &output);
     // "ｎｙ"
     EXPECT_EQ("\xef\xbd\x8e\xef\xbd\x99", output);
 
-    comp.GetStringWithTrimMode(TRIM, &output);
+    composition_->GetStringWithTrimMode(TRIM, &output);
     // ""
     EXPECT_EQ("", output);
   }
@@ -1222,27 +1214,26 @@ TEST_F(CompositionTest, Issue2990253) {
   // "びょ"
   table.AddRule("byo", "\xe3\x81\xb3\xe3\x82\x87", "");
 
-  Composition comp;
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
-  comp.SetInputMode(kHiraganaT12r);
+  composition_->SetInputMode(kHiraganaT12r);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "n");
-  pos = comp.InsertAt(pos, "y");
+  pos = composition_->InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "y");
   pos = 1;
-  pos = comp.InsertAt(pos, "b");
+  pos = composition_->InsertAt(pos, "b");
   {
     string output;
-    comp.GetStringWithTrimMode(FIX, &output);
+    composition_->GetStringWithTrimMode(FIX, &output);
     // "んｂｙ"
     EXPECT_EQ("\xe3\x82\x93\xef\xbd\x82\xef\xbd\x99", output);
 
-    comp.GetStringWithTrimMode(ASIS, &output);
+    composition_->GetStringWithTrimMode(ASIS, &output);
     // "んｂｙ"
     EXPECT_EQ("\xe3\x82\x93\xef\xbd\x82\xef\xbd\x99", output);
 
-    comp.GetStringWithTrimMode(TRIM, &output);
+    composition_->GetStringWithTrimMode(TRIM, &output);
     // "んｂ"
     // doubtful result. should be "ん"
     // May relate to http://b/2990358
@@ -1268,29 +1259,28 @@ TEST_F(CompositionTest, InsertionIntoPreeditMakesInvalidText_1) {
   // "びょ"
   table.AddRule("byo", "\xe3\x81\xb3\xe3\x82\x87", "");
 
-  Composition comp;
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
-  comp.SetInputMode(kHiraganaT12r);
+  composition_->SetInputMode(kHiraganaT12r);
 
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "n");
-  pos = comp.InsertAt(pos, "y");
+  pos = composition_->InsertAt(pos, "n");
+  pos = composition_->InsertAt(pos, "y");
   pos = 1;
-  pos = comp.InsertAt(pos, "b");
+  pos = composition_->InsertAt(pos, "b");
   pos = 3;
-  pos = comp.InsertAt(pos, "o");
+  pos = composition_->InsertAt(pos, "o");
   {
     string output;
-    comp.GetStringWithTrimMode(FIX, &output);
+    composition_->GetStringWithTrimMode(FIX, &output);
     // "んびょ"
     EXPECT_EQ("\xe3\x82\x93\xe3\x81\xb3\xe3\x82\x87", output);
 
-    comp.GetStringWithTrimMode(ASIS, &output);
+    composition_->GetStringWithTrimMode(ASIS, &output);
     // "んびょ"
     EXPECT_EQ("\xe3\x82\x93\xe3\x81\xb3\xe3\x82\x87", output);
 
-    comp.GetStringWithTrimMode(TRIM, &output);
+    composition_->GetStringWithTrimMode(TRIM, &output);
     // "んびょ"
     EXPECT_EQ("\xe3\x82\x93\xe3\x81\xb3\xe3\x82\x87", output);
   }
@@ -1300,32 +1290,31 @@ TEST_F(CompositionTest, InsertionIntoPreeditMakesInvalidText_2) {
   // http://b/2990358
   // Test for mainly Composition::InsertKeyAndPreeditAt()
 
-  Composition comp;
   Table table;
   // "す゛", "ず"
   table.AddRule("\xe3\x81\x99\xe3\x82\x9b", "\xe3\x81\x9a", "");
   // "く゛", "ぐ"
   table.AddRule("\xe3\x81\x8f\xe3\x82\x9b", "\xe3\x81\x90", "");
-  comp.SetTable(&table);
+  composition_->SetTableForUnittest(&table);
 
   size_t pos = 0;
   // "も"
-  pos = comp.InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "m", "\xe3\x82\x82");
   // "す"
-  pos = comp.InsertKeyAndPreeditAt(pos, "r", "\xe3\x81\x99");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "r", "\xe3\x81\x99");
   // "く"
-  pos = comp.InsertKeyAndPreeditAt(pos, "h", "\xe3\x81\x8f");
+  pos = composition_->InsertKeyAndPreeditAt(pos, "h", "\xe3\x81\x8f");
   // "゛"
-  pos = comp.InsertKeyAndPreeditAt(2, "@", "\xe3\x82\x9b");
-  pos = comp.InsertKeyAndPreeditAt(5, "!", "!");
+  pos = composition_->InsertKeyAndPreeditAt(2, "@", "\xe3\x82\x9b");
+  pos = composition_->InsertKeyAndPreeditAt(5, "!", "!");
 
   string comp_str;
-  comp.GetString(&comp_str);
+  composition_->GetString(&comp_str);
   // "もずく!"
   EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a\xe3\x81\x8f\x21", comp_str);
 
   string comp_ascii_str;
-  comp.GetStringWithTransliterator(kRawT12r, &comp_ascii_str);
+  composition_->GetStringWithTransliterator(kRawT12r, &comp_ascii_str);
   EXPECT_EQ("mr@h!", comp_ascii_str);
 }
 
@@ -1346,8 +1335,7 @@ TEST_F(CompositionTest, CombinePendingChunks) {
 
   {
     // empty chunks + "n" -> empty chunks + "n"
-    Composition comp;
-    comp.SetTable(&table);
+    Composition comp(&table);
     comp.SetInputMode(kHiraganaT12r);
 
     size_t pos = 0;
@@ -1367,8 +1355,7 @@ TEST_F(CompositionTest, CombinePendingChunks) {
   {
     // [x] + "n" -> [x] + "n"
     // No combination performed.
-    Composition comp;
-    comp.SetTable(&table);
+    Composition comp(&table);
     comp.SetInputMode(kHiraganaT12r);
 
     size_t pos = 0;
@@ -1389,8 +1376,7 @@ TEST_F(CompositionTest, CombinePendingChunks) {
   {
     // Append "a" to [n][y] -> [ny] + "a"
     // Combination performed.
-    Composition comp;
-    comp.SetTable(&table);
+    Composition comp(&table);
     comp.SetInputMode(kHiraganaT12r);
 
     size_t pos = 0;
@@ -1413,8 +1399,8 @@ TEST_F(CompositionTest, CombinePendingChunks) {
   {
     // Append "a" to [x][n][y] -> [x][ny] + "a"
     // Combination performed.
-    Composition comp;
-    comp.SetTable(&table);
+    Composition comp(&table);
+    comp.SetTableForUnittest(&table);
     comp.SetInputMode(kHiraganaT12r);
 
     size_t pos = 0;
@@ -1440,8 +1426,7 @@ TEST_F(CompositionTest, CombinePendingChunks) {
     // Append "a" of conversion value to [x][n][y] -> [x][ny] + "a"
     // Combination performed.  If composition input contains a
     // conversion, the conversion is used rather than a raw value.
-    Composition comp;
-    comp.SetTable(&table);
+    Composition comp(&table);
     comp.SetInputMode(kHiraganaT12r);
 
     size_t pos = 0;
@@ -1475,17 +1460,16 @@ TEST_F(CompositionTest, AlphanumericOfSSH) {
   // "し"
   table.AddRule("shi", "\xE3\x81\x97", "");
 
-  Composition comp;
-  comp.SetTable(&table);
-  comp.SetInputMode(kHiraganaT12r);
+  composition_->SetTableForUnittest(&table);
+  composition_->SetInputMode(kHiraganaT12r);
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "s");
-  pos = comp.InsertAt(pos, "s");
-  pos = comp.InsertAt(pos, "h");
+  pos = composition_->InsertAt(pos, "s");
+  pos = composition_->InsertAt(pos, "s");
+  pos = composition_->InsertAt(pos, "h");
   EXPECT_EQ(3, pos);
 
   string output;
-  comp.GetStringWithTrimMode(FIX, &output);
+  composition_->GetStringWithTrimMode(FIX, &output);
   // "っｓｈ"
   EXPECT_EQ("\xE3\x81\xA3\xEF\xBD\x93\xEF\xBD\x88", output);
 }
@@ -1498,22 +1482,21 @@ TEST_F(CompositionTest, GrassHack) {
   table.AddRule("we", "\xE3\x81\x86\xE3\x81\x87", "");
   table.AddRule("www", "w", "ww");
 
-  Composition comp;
-  comp.SetTable(&table);
-  comp.SetInputMode(kHiraganaT12r);
+  composition_->SetTableForUnittest(&table);
+  composition_->SetInputMode(kHiraganaT12r);
   size_t pos = 0;
-  pos = comp.InsertAt(pos, "w");
-  pos = comp.InsertAt(pos, "w");
-  pos = comp.InsertAt(pos, "w");
+  pos = composition_->InsertAt(pos, "w");
+  pos = composition_->InsertAt(pos, "w");
+  pos = composition_->InsertAt(pos, "w");
 
   // "ｗｗｗ"
   EXPECT_EQ("\xEF\xBD\x97" "\xEF\xBD\x97" "\xEF\xBD\x97",
-            GetString(comp));
+            GetString(*composition_));
 
-  pos = comp.InsertAt(pos, "e");
+  pos = composition_->InsertAt(pos, "e");
   // "ｗっうぇ"
   EXPECT_EQ("\xEF\xBD\x97" "\xE3\x81\xA3" "\xE3\x81\x86\xE3\x81\x87",
-            GetString(comp));
+            GetString(*composition_));
 }
 
 TEST_F(CompositionTest, RulesForFirstKeyEvents) {
@@ -1611,5 +1594,33 @@ TEST_F(CompositionTest, SetTransliteratorOnEmpty) {
   EXPECT_EQ(1, composition_->GetLength());
 }
 
+TEST_F(CompositionTest, Clone) {
+  Composition src(table_.get());
+  src.SetInputMode(kFullKatakanaT12r);
+
+  // "も"
+  AppendChunk("\xe3\x82\x82", "", "mo", &src);
+  // "ず"
+  AppendChunk("\xe3\x81\x9a", "", "z", &src);
+  // "く"
+  AppendChunk("\xe3\x81\x8f", "", "c", &src);
+
+  EXPECT_TRUE(table_.get() == src.table_);
+  EXPECT_TRUE(kFullKatakanaT12r == src.input_t12r_);
+  EXPECT_EQ(3, src.chunks_.size());
+
+  scoped_ptr<Composition> dest(src.CloneImpl());
+  ASSERT_TRUE(dest.get());
+  EXPECT_TRUE(src.table_ == dest->table_);
+  EXPECT_TRUE(src.input_t12r_ == dest->input_t12r_);
+  ASSERT_EQ(src.chunks_.size(), dest->chunks_.size());
+
+  CharChunkList::iterator src_it = src.chunks_.begin();
+  CharChunkList::iterator dest_it = dest->chunks_.begin();
+  for (; src_it != src.chunks_.end(); ++src_it, ++dest_it) {
+    EXPECT_EQ((*src_it)->raw(), (*dest_it)->raw());
+  }
+}
+
 }  // namespace composer
 }  // namespace mozc
