@@ -186,9 +186,7 @@ void ConfigHandlerImpl::SetImposedConfig(const Config &config) {
 // Reload from file
 bool ConfigHandlerImpl::Reload() {
   VLOG(1) << "Reloading config file: " << filename_;
-  scoped_ptr<istream> is(ConfigFileStream::Open(
-                             filename_,
-                             ios::in | ios::binary));
+  scoped_ptr<istream> is(ConfigFileStream::OpenReadBinary(filename_));
   Config input_proto;
   bool ret_code = true;
 
