@@ -100,6 +100,10 @@ class Segment {
     // The style is defined in enum |Style|.
     uint32 style;
 
+    // Command of this candidate. This is not a bit-field.
+    // The style is defined in enum |Command|.
+    uint32 command;
+
     void Init() {
       key.clear();
       value.clear();
@@ -118,11 +122,12 @@ class Segment {
       usage_id = 0;
       attributes = 0;
       style = 0;
+      command = 0;
     }
 
     Candidate() : cost(0), wcost(0), structure_cost(0),
                   lid(0), rid(0),
-                  attributes(0), style(0) {}
+                  attributes(0), style(0), command(0) {}
 
     enum Attribute {
       DEFAULT_ATTRIBUTE = 0,
@@ -140,6 +145,14 @@ class Segment {
       COMMAND_CANDIDATE = 1024,    // command candidate. e.g., incognito mode.
     };
 
+    enum COMMAND {
+      DEFAULT_COMMAND = 0,
+      ENABLE_INCOGNITO_MODE,      // enables "incognito mode".
+      DISABLE_INCOGNITO_MODE,     // disables "incognito mode".
+      ENABLE_PRESENTATION_MODE,   // enables "presentation mode".
+      DISABLE_PRESENTATION_MODE,  // disables "presentation mode".
+    };
+
     enum Style {
       DEFAULT_STYLE= 0,
       NUMBER_SEPARATED_ARABIC_HALFWIDTH,
