@@ -44,6 +44,10 @@ class DictionaryImpl : public DictionaryInterface {
  public:
   virtual ~DictionaryImpl();
 
+  virtual Node *LookupPredictiveWithLimit(
+      const char *str, int size, const Limit &limit,
+      NodeAllocatorInterface *allocator) const;
+
   virtual Node *LookupPredictive(const char *str, int size,
                                  NodeAllocatorInterface *allocator) const;
 
@@ -52,6 +56,10 @@ class DictionaryImpl : public DictionaryInterface {
       const Limit &limit,
       NodeAllocatorInterface *allocator) const;
 
+  virtual Node *LookupPrefix(
+      const char *str, int size,
+      NodeAllocatorInterface *allocator) const;
+
   virtual Node *LookupReverse(const char *str, int size,
                               NodeAllocatorInterface *allocator) const;
 
@@ -82,7 +90,7 @@ class DictionaryImpl : public DictionaryInterface {
                        const Limit &limit,
                        NodeAllocatorInterface *allocator) const;
 
-  Node *MaybeRemvoeSpecialNodes(Node *node) const;
+  Node *MaybeRemoveSpecialNodes(Node *node) const;
 };
 
 }  // namespace mozc
