@@ -46,6 +46,9 @@
 #include "config/config_handler.h"
 #include "config/config.pb.h"
 #include "config/stats_config_util.h"
+#ifdef ENABLE_CLOUD_SYNC
+#include "gui/config_dialog/auth_dialog.h"
+#endif  // ENABLE_CLOUD_SYNC
 #include "gui/config_dialog/keymap_editor.h"
 #include "gui/config_dialog/roman_table_editor.h"
 #include "gui/base/win_util.h"
@@ -80,11 +83,25 @@ ConfigDialog::ConfigDialog()
 #endif  // OS_MACOSX || OS_LINUX
 #endif  // NO_LOGGING
 
-#if !defined(GOOGLE_JAPANESE_INPUT_BUILD)
-  // Currently "Sync" feature is enabled on official build only.
-  const int kSyncTabIndex = 5;
-  configDialogTabWidget->removeTab(kSyncTabIndex);
-#endif  // !GOOGLE_JAPANESE_INPUT_BUILD
+#ifndef ENABLE_CLOUD_SYNC
+  syncDescriptionLabel->setVisible(false);
+  clearSyncButton->setVisible(false);
+  removeAllDescriptionLabel->setVisible(false);
+  syncStatusLabel->setVisible(false);
+  syncButtonsLayoutWidget->setVisible(false);
+  syncHeaderLayoutWidget->setVisible(false);
+#endif  // !ENABLE_CLOUD_SYNC
+
+#ifndef ENABLE_CLOUD_HANDWRITING
+  cloudHandwritingCheckBox->setVisible(false);
+  cloudServersLayoutWidget->setVisible(false);
+#endif  // !ENABLE_CLOUD_HANDWRITING
+
+#if !defined(ENABLE_CLOUD_SYNC) && !defined(ENABLE_CLOUD_HANDWRITING)
+  // Hide "Cloud" tab when all the cloud features are not available.
+  const int kCloudTabIndex = 5;
+  configDialogTabWidget->removeTab(kCloudTabIndex);
+#endif  // !ENABLE_CLOUD_SYNC && !ENABLE_CLOUD_HANDWRITING
 
   suggestionsSizeSpinBox->setRange(1, 9);
 
@@ -230,6 +247,21 @@ ConfigDialog::ConfigDialog()
                    this,
                    SLOT(LaunchAdministrationDialog()));
 
+#ifdef ENABLE_CLOUD_SYNC
+  QObject::connect(syncToggleButton,
+                   SIGNAL(clicked()),
+                   this,
+                   SLOT(SyncToggleButtonClicked()));
+  QObject::connect(syncCustomizationButton,
+                   SIGNAL(clicked()),
+                   this,
+                   SLOT(LaunchSyncCustomizationDialog()));
+  QObject::connect(clearSyncButton,
+                   SIGNAL(clicked()),
+                   this,
+                   SLOT(ClearSyncClicked()));
+#endif  // ENABLE_CLOUD_SYNC
+
   // When clicking these messages, CheckBoxs corresponding
   // to them should be toggled.
   // We cannot use connect/slot as QLabel doesn't define
@@ -285,9 +317,33 @@ ConfigDialog::ConfigDialog()
   usageStatsCheckBox->setVisible(false);
 #endif // OS_LINUX
 
+  webUsageDictionaryCheckBox->setVisible(false);
+  editWebServiceEntryButton->setVisible(false);
+
+#ifdef ENABLE_CLOUD_SYNC
+  sync_customize_dialog_.reset(new SyncCustomizeDialog(this));
+  sync_running_ = false;
+  // We do not wait with NamedEventListener to update the sync status,
+  // but just do polling every second with QTimer.  If we introduce
+  // NamedEventListener in a watcher thread, the main thread has to
+  // cancel the listener when the window is closed.  That case would
+  // cause another tricky threading issue.
+  timer_.reset(new QTimer(this));
+  QObject::connect(
+      timer_.get(), SIGNAL(timeout()), this, SLOT(UpdateSyncStatus()));
+  timer_->start(1000);
+  last_synced_timestamp_ = 0;
+  // all sync buttons are not enabled for the first time.
+  clearSyncButton->setEnabled(false);
+  syncCustomizationButton->setEnabled(false);
+  syncToggleButton->setEnabled(false);
+#endif  // ENABLE_CLOUD_SYNC
 
   Reload();
 
+  editWebServiceEntryButton->setEnabled(
+      webUsageDictionaryCheckBox->isChecked());
+
 #ifdef OS_WINDOWS
   IMEHotKeyDisabledCheckBox->setChecked(WinUtil::GetIMEHotKeyDisabled());
 #else
@@ -300,6 +356,9 @@ ConfigDialog::ConfigDialog()
 }
 
 ConfigDialog::~ConfigDialog() {
+#ifdef ENABLE_CLOUD_SYNC
+  timer_->stop();
+#endif  // ENABLE_CLOUD_SYNC
 }
 
 bool ConfigDialog::SetConfig(const config::Config &config) {
@@ -493,6 +552,11 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
   SET_CHECKBOX(zipcodeDictionaryCheckBox, use_zip_code_conversion);
   SET_CHECKBOX(spellingCorrectionCheckBox, use_spelling_correction);
 
+  // InfoListConfig
+  localUsageDictionaryCheckBox->setChecked(
+      config.information_list_config().use_local_usage_dictionary());
+  information_list_config_.CopyFrom(config.information_list_config());
+
   // tab3
   SET_CHECKBOX(useAutoImeTurnOff, use_auto_ime_turn_off);
 
@@ -535,6 +599,17 @@ void ConfigDialog::ConvertFromProto(const config::Config &config) {
   SET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Load(config);
+#ifdef ENABLE_CLOUD_SYNC
+  sync_customize_dialog_->Load(config);
+  if (config.has_sync_config()) {
+    sync_running_ = true;
+    syncStatusLabel->setText(tr("Checking sync status"));
+  } else {
+    syncStatusLabel->setText(tr("Sync is not enabled"));
+  }
+  UpdateSyncToggleButtonText();
+#endif  // ENABLE_CLOUD_SYNC
+  SET_CHECKBOX(cloudHandwritingCheckBox, allow_cloud_handwriting);
 }
 
 void ConfigDialog::ConvertToProto(config::Config *config) const {
@@ -571,6 +646,12 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   GET_CHECKBOX(zipcodeDictionaryCheckBox, use_zip_code_conversion);
   GET_CHECKBOX(spellingCorrectionCheckBox, use_spelling_correction);
 
+  // InformationListConfig
+  config->mutable_information_list_config()->CopyFrom(
+      information_list_config_);
+  config->mutable_information_list_config()->set_use_local_usage_dictionary(
+      localUsageDictionaryCheckBox->isChecked());
+
   // tab3
   GET_CHECKBOX(useAutoImeTurnOff, use_auto_ime_turn_off);
 
@@ -620,6 +701,10 @@ void ConfigDialog::ConvertToProto(config::Config *config) const {
   GET_COMBOBOX(yenSignComboBox, YenSignCharacter, yen_sign_character);
 
   characterFormEditor->Save(config);
+#ifdef ENABLE_CLOUD_SYNC
+  sync_customize_dialog_->Save(sync_running_, config);
+#endif  // ENABLE_CLOUD_SYNC
+  GET_CHECKBOX(cloudHandwritingCheckBox, allow_cloud_handwriting);
 }
 
 #undef SET_COMBOBOX
@@ -727,7 +812,7 @@ void ConfigDialog::EditKeymap() {
     const char *keymap_file =
         keymap::KeyMapManager::GetKeyMapFileName(itr->second);
     scoped_ptr<istream> ifs(
-        ConfigFileStream::Open(keymap_file));
+        ConfigFileStream::LegacyOpen(keymap_file));
     CHECK(ifs.get() != NULL);  // should never happen
     stringstream buffer;
     buffer << ifs->rdbuf();
@@ -754,6 +839,9 @@ void ConfigDialog::EditRomanTable() {
   }
 }
 
+void ConfigDialog::EditInforlistConfig() {
+}
+
 void ConfigDialog::SelectInputModeSetting(int index) {
   // enable "EDIT" button if roman mode is selected
   editRomanTableButton->setEnabled((index == 0));
@@ -776,6 +864,29 @@ void ConfigDialog::SelectSuggestionSetting(int state) {
   }
 }
 
+void ConfigDialog::SelectWebUsageDictionarySetting(bool checked) {
+  if (checked) {
+    if (QMessageBox::question(
+            this,
+            tr("Mozc settings"),
+            tr("Web Service extension enables Mozc "
+               "to display third party usage dictionaries provided as an "
+               "Web service API, e.g. REST (POX/JSON over HTTP). "
+               "Note that candidate/preedit strings are "
+               "sent to the Web service when this feature is enabled. "
+               "Do you want to use Web Service extension?"),
+            QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
+        == QMessageBox::Yes) {
+      editWebServiceEntryButton->setEnabled(true);
+    } else {
+      webUsageDictionaryCheckBox->setChecked(false);
+      editWebServiceEntryButton->setEnabled(false);
+    }
+  } else {
+    editWebServiceEntryButton->setEnabled(false);
+  }
+}
+
 void ConfigDialog::ResetToDefaults() {
   if (QMessageBox::Ok ==
       QMessageBox::question(
@@ -803,9 +914,283 @@ void ConfigDialog::ResetToDefaults() {
 void ConfigDialog::LaunchAdministrationDialog() {
 #ifdef OS_WINDOWS
   client_->LaunchTool("administration_dialog", "");
-#endif
+#endif  // OS_WINDOWS
 }
 
+void ConfigDialog::SyncToggleButtonClicked() {
+#ifdef ENABLE_CLOUD_SYNC
+  SyncToggleButtonClickedImpl();
+#endif  // ENABLE_CLOUD_SYNC
+}
+
+void ConfigDialog::LaunchSyncCustomizationDialog() {
+#ifdef ENABLE_CLOUD_SYNC
+  LaunchSyncCustomizationDialogImpl();
+#endif  // ENABLE_CLOUD_SYNC
+}
+
+void ConfigDialog::UpdateSyncStatus() {
+#ifdef ENABLE_CLOUD_SYNC
+  UpdateSyncStatusImpl();
+#endif  // ENABLE_CLOUD_SYNC
+}
+
+void ConfigDialog::ClearSyncClicked() {
+#ifdef ENABLE_CLOUD_SYNC
+  ClearSyncClickedImpl();
+#endif  // ENABLE_CLOUD_SYNC
+}
+
+#ifdef ENABLE_CLOUD_SYNC
+void ConfigDialog::UpdateSyncToggleButtonText() {
+  if (sync_running_) {
+    syncToggleButton->setText(tr("Stop Sync"));
+  } else {
+    syncToggleButton->setText(tr("Start Sync"));
+  }
+}
+
+void ConfigDialog::SyncToggleButtonClickedImpl() {
+  if (sync_running_) {
+    StopSync();
+  } else {
+    LaunchAuthDialog();
+  }
+}
+
+void ConfigDialog::StopSync() {
+  if (QMessageBox::Cancel ==
+      QMessageBox::question(
+          this,
+          tr("Stop sync and reset the auth token"),
+          tr("You are trying to stop sync and reset the auth token. "
+             "To restart sync, you will need to get a new auth token.\n"
+             "Do you really want to continue?"),
+          QMessageBox::Ok | QMessageBox::Cancel,
+          QMessageBox::Cancel)) {
+    return;
+  }
+
+  // To stop the current synchronization, it resets the authorization code.
+  commands::Input::AuthorizationInfo auth_info;
+  auth_info.set_auth_code("");
+  if (!client_->AddAuthCode(auth_info)) {
+    LOG(WARNING) << "Sending auth code failed";
+    return;
+  }
+  sync_running_ = false;
+  UpdateSyncToggleButtonText();
+}
+
+void ConfigDialog::LaunchAuthDialog() {
+  string auth_code;
+  if (!gui::AuthDialog::Show(this, &auth_code)) {
+    LOG(WARNING) << "Canceled.  Do nothing";
+    return;
+  }
+
+  // The default configuration at the starting timing will be "all".
+  config::Config mock_config;
+  config::SyncConfig *sync_config = mock_config.mutable_sync_config();
+  sync_config->set_use_config_sync(true);
+  sync_config->set_use_user_dictionary_sync(true);
+  sync_config->set_use_user_history_sync(true);
+  sync_config->set_use_learning_preference_sync(true);
+  sync_config->set_use_contact_list_sync(true);
+
+  // Store the current config for the rollback when failed.
+  config::Config current_config;
+  sync_customize_dialog_->Save(false, &current_config);
+  sync_customize_dialog_->Load(mock_config);
+
+  // Then ask users to customize the sync configuration.
+  if (QDialog::Accepted != sync_customize_dialog_->exec()) {
+    LOG(WARNING) << "Canceled during sync customization. "
+                 << "It means do not activate sync.";
+    // Rollback the config.
+    sync_customize_dialog_->Load(current_config);
+    return;
+  }
+
+  sync_running_ = true;
+  // Before starting sync, the sync customization should be applied.
+  // WARNING: the whole configuration (items other than sync) should
+  // be applied at the same time.
+  Update();
+
+  commands::Input::AuthorizationInfo auth_info;
+  auth_info.set_auth_code(auth_code);
+  if (!client_->AddAuthCode(auth_info)) {
+    LOG(WARNING) << "Sending auth code failed";
+    return;
+  }
+  if (!client_->StartCloudSync()) {
+    LOG(WARNING) << "Cloud sync is failed to start";
+  }
+  UpdateSyncToggleButtonText();
+  // It does not wait for the end of cloud sync.
+}
+
+void ConfigDialog::LaunchSyncCustomizationDialogImpl() {
+  if (QDialog::Accepted == sync_customize_dialog_->exec()) {
+    // Before starting sync, the sync customization should be applied.
+    // WARNING: the whole configuration (items other than sync) should
+    // be applied at the same time.
+    Update();
+    if (!client_->StartCloudSync()) {
+      LOG(WARNING) << "Cloud sync is failed to start";
+    }
+  }
+}
+
+void ConfigDialog::AddLastSyncedDateTime(
+    uint64 timestamp, QString *output) const {
+  // 0 is default value, and means to have no sync before.
+  if (timestamp == 0) {
+    output->append(tr("Not synced yet"));
+    return;
+  }
+
+  output->append(tr("Last synced time: "));
+  const int time_offset = Util::GetTime() - timestamp;
+  QDateTime datetime = QDateTime::currentDateTime().addSecs(-time_offset);
+  output->append(datetime.toString("yyyy/MM/dd hh:mm:ss"));
+}
+
+void ConfigDialog::UpdateSyncStatusImpl() {
+  // This method always updates the sync status even when sync is not
+  // running.  That's because we should change the message to "not
+  // enabled" when the user explicitly turns off sync feature.
+  QString sync_message;
+  commands::CloudSyncStatus cloud_sync_status;
+  // The default status is 'NOSYNC'.  Only changed via IPC when sync
+  // is running.
+  cloud_sync_status.set_global_status(commands::CloudSyncStatus::NOSYNC);
+  if (sync_running_) {
+    client_->GetCloudSyncStatus(&cloud_sync_status);
+    DLOG(INFO) << cloud_sync_status.DebugString();
+  }
+
+  commands::CloudSyncStatus::SyncGlobalStatus sync_global_status =
+      cloud_sync_status.global_status();
+
+  // clearing sync is only available when sync is succeeding ==
+  // the current authorization is valid.
+  clearSyncButton->setEnabled(
+      sync_global_status == commands::CloudSyncStatus::SYNC_SUCCESS);
+  syncCustomizationButton->setEnabled(true);
+  syncToggleButton->setEnabled(true);
+
+  switch (sync_global_status) {
+    case commands::CloudSyncStatus::SYNC_SUCCESS:
+    case commands::CloudSyncStatus::SYNC_FAILURE: {
+      uint64 last_synced_timestamp = cloud_sync_status.last_synced_timestamp();
+      AddLastSyncedDateTime(last_synced_timestamp, &sync_message);
+      if (last_synced_timestamp_ != last_synced_timestamp) {
+        Reload();
+        last_synced_timestamp_ = last_synced_timestamp;
+      }
+      break;
+    }
+    case commands::CloudSyncStatus::WAITSYNC:
+      sync_message = tr("Waiting for server to be ready");
+      syncCustomizationButton->setEnabled(false);
+      syncToggleButton->setEnabled(false);
+      clearSyncButton->setEnabled(false);
+      break;
+    case commands::CloudSyncStatus::INSYNC:
+      sync_message = tr("During synchronization");
+      syncCustomizationButton->setEnabled(false);
+      syncToggleButton->setEnabled(false);
+      clearSyncButton->setEnabled(false);
+      break;
+    case commands::CloudSyncStatus::NOSYNC:
+      sync_message = tr("Sync is not enabled");
+      sync_running_ = false;
+      syncCustomizationButton->setEnabled(false);
+      clearSyncButton->setEnabled(false);
+      break;
+  }
+
+  QString tooltip;
+  // Check error messages in sync.
+  if (cloud_sync_status.sync_errors_size() > 0) {
+    for (size_t i = 0; i < cloud_sync_status.sync_errors_size(); ++i) {
+      if (i > 0) {
+        tooltip += "\n";
+      }
+
+      switch (cloud_sync_status.sync_errors(i).error_code()) {
+        case commands::CloudSyncStatus::AUTHORIZATION_FAIL:
+          tooltip += tr("Authorization failed.");
+          break;
+        case commands::CloudSyncStatus::USER_DICTIONARY_NUM_ENTRY_EXCEEDED:
+          tooltip += tr("Cannot save dictionaries because Sync Dictionary "
+                        "exceeds its entry size limit.");
+          break;
+        case commands::CloudSyncStatus::USER_DICTIONARY_BYTESIZE_EXCEEDED:
+          tooltip += tr("Cannot save dictionaries because Sync Dictionary "
+                        "exceeds its binary size limit.");
+          break;
+        case commands::CloudSyncStatus::USER_DICTIONARY_NUM_DICTIONARY_EXCEEDED:
+          tooltip += tr("Cannot save dictionaries because the number of "
+                        "dictionaries for sync exceeds its limit.");
+          break;
+        default:
+          tooltip += tr("Unknown error was found.");
+          LOG(WARNING) << "Unknown error ("
+                       << cloud_sync_status.sync_errors(i).error_code()
+                       << ") was found in cloud sync errors.";
+          break;
+      }
+    }
+
+    sync_message += "  ";
+    // Display the first error category obviously.
+    switch (cloud_sync_status.sync_errors(0).error_code()) {
+      case commands::CloudSyncStatus::AUTHORIZATION_FAIL:
+        sync_message += tr("Authorization error");
+        break;
+      case commands::CloudSyncStatus::USER_DICTIONARY_NUM_ENTRY_EXCEEDED:
+      case commands::CloudSyncStatus::USER_DICTIONARY_BYTESIZE_EXCEEDED:
+      case commands::CloudSyncStatus::USER_DICTIONARY_NUM_DICTIONARY_EXCEEDED:
+        sync_message += tr("Dictionary sync error");
+        break;
+      default:
+        sync_message += tr("Unknown sync error");
+        break;
+    }
+  }
+
+  UpdateSyncToggleButtonText();
+  syncStatusLabel->setText(sync_message);
+  syncStatusLabel->setToolTip(tooltip);
+}
+
+void ConfigDialog::ClearSyncClickedImpl() {
+  if (QMessageBox::Cancel ==
+      QMessageBox::question(
+          this,
+          tr("Clear all sync data in Google servers"),
+          tr("You are trying to clear all the data in the sync server.\n"
+             "Do you really want to continue?"),
+          QMessageBox::Ok | QMessageBox::Cancel,
+          QMessageBox::Cancel)) {
+    return;
+  }
+
+  // Stop doing sync configurations.
+  config::Config config;
+  GetConfig(&config);
+  config.clear_sync_config();
+  sync_running_ = false;
+  ConvertFromProto(config);
+  Update();
+
+  // Do the actual clear.
+  client_->ClearCloudSync();
+}
+#endif  // ENABLE_CLOUD_SYNC
 
 // Catch MouseButtonRelease event to toggle the CheckBoxes
 bool ConfigDialog::eventFilter(QObject *obj, QEvent *event) {
@@ -820,5 +1205,6 @@ bool ConfigDialog::eventFilter(QObject *obj, QEvent *event) {
   }
   return QObject::eventFilter(obj, event);
 }
+
 }  // namespace gui
 }  // namespace mozc
