@@ -51,6 +51,8 @@ int RunDictionaryTool(int argc, char *argv[]) {
     return -1;
   }
 
+  // TODO(nona): remove these lines when link-time language dependency
+  //             injection is rolled out.
   mozc::japanese::LangDepSpecJapanese spec;
   mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(&spec);
 
@@ -64,5 +66,12 @@ int RunDictionaryTool(int argc, char *argv[]) {
 
   window.show();
   window.raise();
-  return app.exec();
+
+  const int result = app.exec();
+
+  // TODO(nona): remove this line when link-time language dependency
+  //             injection is rolled out.
+  mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(NULL);
+
+  return result;
 }
