@@ -31,6 +31,8 @@
 
 #include "base/password_manager.h"
 #include "base/util.h"
+#include "composer/composer.h"
+#include "composer/table.h"
 #include "config/config.pb.h"
 #include "config/config_handler.h"
 #include "converter/converter_interface.h"
@@ -44,6 +46,7 @@
 
 
 DECLARE_string(test_tmpdir);
+DECLARE_bool(enable_expansion_for_user_history_predictor);
 
 namespace mozc {
 namespace {
@@ -125,6 +128,14 @@ void PrependHistorySegments(const string &key,
 }   // anonymous namespace
 
 class UserHistoryPredictorTest : public testing::Test {
+ public:
+  UserHistoryPredictorTest() :
+      default_expansion_(FLAGS_enable_expansion_for_user_history_predictor) {}
+
+  virtual ~UserHistoryPredictorTest() {
+    FLAGS_enable_expansion_for_user_history_predictor = default_expansion_;
+  }
+
  protected:
   virtual void SetUp() {
     kUseMockPasswordManager = true;
@@ -135,6 +146,7 @@ class UserHistoryPredictorTest : public testing::Test {
 
   virtual void TearDown() {
     config::ConfigHandler::SetConfig(default_config_);
+    FLAGS_enable_expansion_for_user_history_predictor = default_expansion_;
   }
 
   virtual void EnableZeroQuerySuggestion() {
@@ -145,6 +157,7 @@ class UserHistoryPredictorTest : public testing::Test {
 
  private:
   config::Config default_config_;
+  const bool default_expansion_;
 };
 
 TEST_F(UserHistoryPredictorTest, UserHistoryPredictorTest) {
@@ -1514,10 +1527,6 @@ struct Command {
   Command() : type(LOOKUP) {}
 };
 
-int Random(int size) {
-  return static_cast<int> (1.0 * size * rand() / (RAND_MAX + 1.0));
-}
-
 TEST_F(UserHistoryPredictorTest, SyncTest) {
   UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
@@ -1526,7 +1535,7 @@ TEST_F(UserHistoryPredictorTest, SyncTest) {
   for (size_t i = 0; i < commands.size(); ++i) {
     commands[i].key = Util::SimpleItoa(i) + "key";
     commands[i].value = Util::SimpleItoa(i) + "value";
-    const int n = Random(100);
+    const int n = Util::Random(100);
     if (n == 0) {
       commands[i].type = Command::WAIT;
     } else if (n < 10) {
@@ -1565,27 +1574,26 @@ TEST_F(UserHistoryPredictorTest, SyncTest) {
 }
 
 TEST_F(UserHistoryPredictorTest, GetMatchTypeTest) {
-  UserHistoryPredictor predictor;
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
-            predictor.GetMatchType("test", ""));
+            UserHistoryPredictor::GetMatchType("test", ""));
 
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
-            predictor.GetMatchType("", ""));
+            UserHistoryPredictor::GetMatchType("", ""));
 
   EXPECT_EQ(UserHistoryPredictor::LEFT_EMPTY_MATCH,
-            predictor.GetMatchType("", "test"));
+            UserHistoryPredictor::GetMatchType("", "test"));
 
   EXPECT_EQ(UserHistoryPredictor::NO_MATCH,
-            predictor.GetMatchType("foo", "bar"));
+            UserHistoryPredictor::GetMatchType("foo", "bar"));
 
   EXPECT_EQ(UserHistoryPredictor::EXACT_MATCH,
-            predictor.GetMatchType("foo", "foo"));
+            UserHistoryPredictor::GetMatchType("foo", "foo"));
 
   EXPECT_EQ(UserHistoryPredictor::LEFT_PREFIX_MATCH,
-            predictor.GetMatchType("foo", "foobar"));
+            UserHistoryPredictor::GetMatchType("foo", "foobar"));
 
   EXPECT_EQ(UserHistoryPredictor::RIGHT_PREFIX_MATCH,
-            predictor.GetMatchType("foobar", "foo"));
+            UserHistoryPredictor::GetMatchType("foobar", "foo"));
 }
 
 TEST_F(UserHistoryPredictorTest, FingerPrintTest) {
@@ -1859,39 +1867,229 @@ TEST_F(UserHistoryPredictorTest, EntryPriorityQueueTest) {
   }
 }
 
+namespace {
+
+string RemoveLastUCS4Character(const string &input) {
+  const char *begin = input.data();
+  const char *end = begin + input.size();
+  const size_t ucs4_count = Util::CharsLen(input);
+  if (ucs4_count == 0) {
+    return "";
+  }
+
+  size_t ucs4_processed = 0;
+  string output;
+  while ((begin < end) && (ucs4_processed < ucs4_count - 1)) {
+    size_t mblen = 0;
+    const char32 ucs4 = Util::UTF8ToUCS4(begin, end, &mblen);
+    Util::UCS4ToUTF8Append(ucs4, &output);
+    begin += mblen;
+    ++ucs4_processed;
+  }
+  return output;
+}
+
+struct PrivacySensitiveTestData {
+  bool is_sensitive;
+  const char *scenario_description;
+  const char *input;
+  const char *output;
+};
+
+const bool kSensitive = true;
+const bool kNonSensitive = false;
+
+const PrivacySensitiveTestData kNonSensitiveCases[] = {
+  {
+    kNonSensitive,  // We might want to revisit this behavior
+    "Type privacy sensitive number but it is commited as full-width number "
+    "by mistake.",
+    "0007",
+    // "０００７"
+    "\xEF\xBC\x90\\xEF\xBC\x90\xEF\xBC\x90\xEF\xBC\x97",
+  }, {
+    kNonSensitive,
+    "Type a ZIP number.",
+    "100-0001",
+    // "東京都千代田区千代田"
+    "\xE6\x9D\xB1\xE4\xBA\xAC\xE9\x83\xBD\xE5\x8D\x83\xE4\xBB\xA3"
+    "\xE7\x94\xB0\xE5\x8C\xBA\xE5\x8D\x83\xE4\xBB\xA3\xE7\x94\xB0"
+  }, {
+    kNonSensitive,  // We might want to revisit this behavior
+    "Type privacy sensitive number but the result contains one or more "
+    "non-ASCII character such as full-width dash.",
+    "1111-1111",
+    // "1111－1111"
+    "1111" "\xEF\xBC\x8D" "1111"
+  }, {
+    kNonSensitive,  // We might want to revisit this behavior
+    "User dictionary contains a credit card number.",
+    // "かーどばんごう"
+    "\xE3\x81\x8B\xE3\x83\xBC\xE3\x81\xA9\xE3\x81\xB0\xE3\x82\x93"
+    "\xE3\x81\x94\xE3\x81\x86",
+    "0000-0000-0000-0000"
+  }, {
+    kNonSensitive,  // We might want to revisit this behavior
+    "User dictionary contains a credit card number.",
+    // "かーどばんごう"
+    "\xE3\x81\x8B\xE3\x83\xBC\xE3\x81\xA9\xE3\x81\xB0\xE3\x82\x93"
+    "\xE3\x81\x94\xE3\x81\x86",
+    "0000000000000000"
+  }, {
+    kNonSensitive,  // We might want to revisit this behavior
+    "User dictionary contains privacy sensitive information.",
+    // "ぱすわーど"
+    "\xE3\x81\xB1\xE3\x81\x99\xE3\x82\x8F\xE3\x83\xBC\xE3\x81\xA9",
+    "ywwz1sxm"
+  }, {
+    kNonSensitive,  // We might want to revisit this behavior
+    "Input privacy sensitive text by Roman-input mode by mistake and then "
+    "hit F10 key to convert it to half-alphanumeric text. In this case "
+    "we assume all the alphabetical characters are consumed by Roman-input "
+    "rules.",
+    // "いあ1ぼ3ぅ"
+    "\343\201\204\343\201\202" "1" "\343\201\274" "3" "\343\201\205",
+    "ia1bo3xu"
+  }, {
+    kNonSensitive,
+    "Katakana to English transliteration.",  // http://b/4394325
+    // "おれんじ"
+      "\xE3\x81\x8A\xE3\x82\x8C\xE3\x82\x93\xE3\x81\x98",
+    "Orange"
+  }, {
+    kNonSensitive,
+    "Input a very common English word which should be included in our "
+    "system dictionary by Roman-input mode by mistake and "
+    "then hit F10 key to convert it to half-alphanumeric text.",
+    // おらんげ"
+    "\xE3\x81\x8A\xE3\x82\x89\xE3\x82\x93\xE3\x81\x92",
+    "orange"
+  }, {
+    kSensitive,
+    "Input a password-like text.",
+    "123abc!",
+    "123abc!",
+  }, {
+    kSensitive,
+    "Input privacy sensitive text by Roman-input mode by mistake and then "
+    "hit F10 key to convert it to half-alphanumeric text. In this case, "
+    "there may remain one or more alphabetical characters, which have not "
+    "been consumed by Roman-input rules.",
+    // "yっwz1sxm"
+    "y" "\343\201\243" "wz1sxm",
+    "ywwz1sxm"
+  }, {
+    kSensitive,  // We might want to revisit this behavior
+    "Type a very common English word which should be included in our "
+    "system dictionary without capitalization.",
+    "those",
+    "those"
+  }, {
+    kSensitive,  // We might want to revisit this behavior
+    "Type a very common English word which should be included in our "
+    "system dictionary with capitalization.",
+    "Those",
+    "Those"
+  }, {
+    kSensitive,  // We might want to revisit this behavior
+    "Type a very common English word which should be included in our "
+    "system dictionary with random capitalizations.",
+    "tHoSe",
+    "tHoSe"
+  }, {
+    kSensitive,  // We might want to revisit this behavior
+    "Type just a number.",
+    "2398402938402934",
+    "2398402938402934"
+  }, {
+    kSensitive,  // We might want to revisit this behavior
+    "Type an common English word which might be included in our system "
+    "dictionary with number postfix.",
+    "Orange10000",
+    "Orange10000"
+  },
+};
+
+}  // namespace
+
 TEST_F(UserHistoryPredictorTest, PrivacySensitiveTest) {
   UserHistoryPredictor predictor;
+
+  for (size_t i = 0; i < arraysize(kNonSensitiveCases); ++i) {
+    predictor.ClearAllHistory();
+    predictor.WaitForSyncer();
+
+    const PrivacySensitiveTestData &data = kNonSensitiveCases[i];
+    const string description(data.scenario_description);
+    const string input(data.input);
+    const string output(data.output);
+    const string &partial_input = RemoveLastUCS4Character(input);
+    const bool expect_sensitive = data.is_sensitive;
+
+    // Initial commit.
+    {
+      Segments segments;
+      MakeSegmentsForConversion(input, &segments);
+      AddCandidate(0, output, &segments);
+      predictor.Finish(&segments);
+    }
+
+    // TODO(yukawa): Refactor the scenario runner below by making
+    //     some utility functions.
+
+    // Check suggestion
+    {
+      Segments segments;
+      MakeSegmentsForSuggestion(partial_input, &segments);
+      if (expect_sensitive) {
+        EXPECT_FALSE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      } else {
+        EXPECT_TRUE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      }
+      segments.Clear();
+      MakeSegmentsForPrediction(input, &segments);
+      if (expect_sensitive) {
+        EXPECT_FALSE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      } else {
+        EXPECT_TRUE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      }
+    }
+
+    // Check Prediction
+    {
+      Segments segments;
+      MakeSegmentsForPrediction(partial_input, &segments);
+      if (expect_sensitive) {
+        EXPECT_FALSE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      } else {
+        EXPECT_TRUE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      }
+      segments.Clear();
+      MakeSegmentsForPrediction(input, &segments);
+      if (expect_sensitive) {
+        EXPECT_FALSE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      } else {
+        EXPECT_TRUE(predictor.Predict(&segments))
+          << description << " input: " << input << " output: " << output;
+      }
+    }
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, PrivacySensitiveMultiSegmentsTest) {
+  UserHistoryPredictor predictor;
   predictor.WaitForSyncer();
 
-  {
-    Segments segments;
-    MakeSegmentsForConversion("123abc!", &segments);
-    AddCandidate("123abc!", &segments);
-    predictor.Finish(&segments);
-  }
-
-  // no suggestion for password-like input
-  {
-    Segments segments;
-    MakeSegmentsForSuggestion("123abc", &segments);
-    EXPECT_FALSE(predictor.Predict(&segments));
-    segments.Clear();
-    MakeSegmentsForSuggestion("123abc!", &segments);
-    EXPECT_FALSE(predictor.Predict(&segments));
-  }
-
-  // no prediction for password-like input
-  {
-    Segments segments;
-    MakeSegmentsForPrediction("123abc", &segments);
-    EXPECT_FALSE(predictor.Predict(&segments));
-    segments.Clear();
-    MakeSegmentsForPrediction("123abc!", &segments);
-    EXPECT_FALSE(predictor.Predict(&segments));
-  }
-
-  predictor.ClearAllHistory();
-  predictor.WaitForSyncer();
+  // If a password-like input consists of multiple segments, it is not
+  // considered to be privacy sensitive when the input is committed.
+  // Currently this is a known issue.
   {
     Segments segments;
     MakeSegmentsForConversion("123", &segments);
@@ -1901,7 +2099,6 @@ TEST_F(UserHistoryPredictorTest, PrivacySensitiveTest) {
     predictor.Finish(&segments);
   }
 
-  // treat as not privacy sensitive but conversion result
   {
     Segments segments;
     MakeSegmentsForSuggestion("123abc", &segments);
@@ -1919,27 +2116,6 @@ TEST_F(UserHistoryPredictorTest, PrivacySensitiveTest) {
     MakeSegmentsForPrediction("123abc!", &segments);
     EXPECT_TRUE(predictor.Predict(&segments));
   }
-
-  predictor.ClearAllHistory();
-  predictor.WaitForSyncer();
-  {
-    Segments segments;
-    // "ぐーぐる"
-    MakeSegmentsForConversion(
-        "\xE3\x81\x90\xE3\x83\xBC\xE3\x81\x90\xE3\x82\x8B",
-        &segments);
-    AddCandidate(0, "Google", &segments);
-    predictor.Finish(&segments);
-  }
-
-  // treat as not privacy sensitive but conversion result
-  {
-    Segments segments;
-    // "ぐーぐ"
-    MakeSegmentsForSuggestion(
-        "\xE3\x81\x90\xE3\x83\xBC\xE3\x81\x90", &segments);
-    EXPECT_TRUE(predictor.Predict(&segments));
-  }
 }
 
 TEST_F(UserHistoryPredictorTest, UserHistoryStorage) {
@@ -1947,7 +2123,6 @@ TEST_F(UserHistoryPredictorTest, UserHistoryStorage) {
       Util::JoinPath(Util::GetUserProfileDirectory(), "test");
 
   UserHistoryStorage storage1(filename);
-  EXPECT_EQ(filename, storage1.filename());
 
   UserHistoryPredictor::Entry *entry = storage1.add_entries();
   CHECK(entry);
@@ -2141,4 +2316,636 @@ TEST_F(UserHistoryPredictorTest, RomanFuzzyLookupEntry) {
   EXPECT_FALSE(predictor.RomanFuzzyLookupEntry("gu-guru", &entry, &results));
   EXPECT_FALSE(predictor.RomanFuzzyLookupEntry("g=guru", &entry, &results));
 }
+
+namespace {
+struct LookupTestData {
+  const string entry_key;
+  const bool expect_result;
+};
+}  // namespace
+
+TEST_F(UserHistoryPredictorTest, ExpandedLookupRoman) {
+  UserHistoryPredictor predictor;
+  UserHistoryPredictor::Entry entry;
+  UserHistoryPredictor::EntryPriorityQueue results;
+
+  // Roman
+  // preedit: "あｋ"
+  // input_key: "あｋ"
+  // key_base: "あ"
+  // key_expanded: "か","き","く","け", "こ"
+  scoped_ptr<Trie<string> > expanded(new Trie<string>);
+  // "か"
+  expanded->AddEntry("\xe3\x81\x8b", "");
+  // "き"
+  expanded->AddEntry("\xe3\x81\x8d", "");
+  // "く"
+  expanded->AddEntry("\xe3\x81\x8f", "");
+  // "け"
+  expanded->AddEntry("\xe3\x81\x91", "");
+  // "こ"
+  expanded->AddEntry("\xe3\x81\x93", "");
+
+  const LookupTestData kTests1[] = {
+    { "", false },
+    // "あか"
+    { "\xe3\x81\x82\xe3\x81\x8b", true },
+    // "あき"
+    { "\xe3\x81\x82\xe3\x81\x8d", true },
+    // "あかい"
+    { "\xe3\x81\x82\xe3\x81\x8b\xe3\x81\x84", true },
+    // "あまい"
+    { "\xe3\x81\x82\xe3\x81\xbe\xe3\x81\x84", false },
+    // "あ"
+    { "\xe3\x81\x82", false },
+    // "さか"
+    { "\xe3\x81\x95\xe3\x81\x8b", false },
+    // "さき"
+    { "\xe3\x81\x95\xe3\x81\x8d", false },
+    // "さかい"
+    { "\xe3\x81\x95\xe3\x81\x8b\xe3\x81\x84", false },
+    // "さまい"
+    { "\xe3\x81\x95\xe3\x81\xbe\xe3\x81\x84", false },
+    // "さ"
+    { "\xe3\x81\x95", false },
+  };
+
+  // with expanded
+  for (size_t i = 0; i < arraysize(kTests1); ++i) {
+    entry.set_key(kTests1[i].entry_key);
+    EXPECT_EQ(kTests1[i].expect_result, predictor.LookupEntry(
+        // "あｋ", "あ"
+        "\xe3\x81\x82\xef\xbd\x8b", "\xe3\x81\x82",
+        expanded.get(), &entry, NULL, &results))
+        << kTests1[i].entry_key;
+  }
+
+  // only expanded
+  // preedit: "k"
+  // input_key: ""
+  // key_base: ""
+  // key_expanded: "か","き","く","け", "こ"
+
+  const LookupTestData kTests2[] = {
+    { "", false },
+    // "か"
+    { "\xe3\x81\x8b", true },
+    // "き"
+    { "\xe3\x81\x8d", true },
+    // "かい"
+    { "\xe3\x81\x8b\xe3\x81\x84", true },
+    // "まい"
+    { "\xe3\x81\xbe\xe3\x81\x84", false },
+    // "も"
+    { "\xe3\x82\x82", false },
+  };
+
+  for (size_t i = 0; i < arraysize(kTests2); ++i) {
+    entry.set_key(kTests2[i].entry_key);
+    EXPECT_EQ(kTests2[i].expect_result, predictor.LookupEntry(
+        "", "", expanded.get(), &entry, NULL, &results))
+        << kTests2[i].entry_key;
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, ExpandedLookupKana) {
+  UserHistoryPredictor predictor;
+  UserHistoryPredictor::Entry entry;
+  UserHistoryPredictor::EntryPriorityQueue results;
+
+  // Kana
+  // preedit: "あし"
+  // input_key: "あし"
+  // key_base: "あ"
+  // key_expanded: "し","じ"
+  scoped_ptr<Trie<string> > expanded(new Trie<string>);
+  // "し"
+  expanded->AddEntry("\xe3\x81\x97", "");
+  // "じ"
+  expanded->AddEntry("\xe3\x81\x98", "");
+
+  const LookupTestData kTests1[] = {
+    { "", false },
+    // "あ"
+    { "\xe3\x81\x82", false },
+    // "あし"
+    { "\xe3\x81\x82\xe3\x81\x97", true },
+    // "あじ"
+    { "\xe3\x81\x82\xe3\x81\x98", true },
+    // "あしかゆい"
+    { "\xe3\x81\x82\xe3\x81\x97\xe3\x81\x8b\xe3\x82\x86\xe3\x81\x84", true },
+    // "あじうまい"
+    { "\xe3\x81\x82\xe3\x81\x98\xe3\x81\x86\xe3\x81\xbe\xe3\x81\x84", true },
+    // "あまにがい"
+    { "\xe3\x81\x82\xe3\x81\xbe\xe3\x81\xab\xe3\x81\x8c\xe3\x81\x84", false },
+    // "あめ"
+    { "\xe3\x81\x82\xe3\x82\x81", false },
+    // "まし"
+    { "\xe3\x81\xbe\xe3\x81\x97", false },
+    // "まじ"
+    { "\xe3\x81\xbe\xe3\x81\x98", false },
+    // "ましなあじ"
+    { "\xe3\x81\xbe\xe3\x81\x97\xe3\x81\xaa\xe3\x81\x82\xe3\x81\x98", false },
+    // "まじうまい"
+    { "\xe3\x81\xbe\xe3\x81\x98\xe3\x81\x86\xe3\x81\xbe\xe3\x81\x84", false },
+    // "ままにがい"
+    { "\xe3\x81\xbe\xe3\x81\xbe\xe3\x81\xab\xe3\x81\x8c\xe3\x81\x84", false },
+    // "まめ"
+    { "\xe3\x81\xbe\xe3\x82\x81", false },
+  };
+
+  // with expanded
+  for (size_t i = 0; i < arraysize(kTests1); ++i) {
+    entry.set_key(kTests1[i].entry_key);
+    EXPECT_EQ(kTests1[i].expect_result, predictor.LookupEntry(
+        // "あし", "あ"
+        "\xe3\x81\x82\xe3\x81\x97", "\xe3\x81\x82",
+        expanded.get(), &entry, NULL, &results))
+        << kTests1[i].entry_key;
+  }
+
+  // only expanded
+  // input_key: "し"
+  // key_base: ""
+  // key_expanded: "し","じ"
+  const LookupTestData kTests2[] = {
+    { "", false },
+    // "し"
+    { "\xe3\x81\x97", true },
+    // "じ"
+    { "\xe3\x81\x98", true },
+    // "しかうまい"
+    { "\xe3\x81\x97\xe3\x81\x8b\xe3\x81\x86\xe3\x81\xbe\xe3\x81\x84", true },
+    // "じゅうかい"
+    { "\xe3\x81\x98\xe3\x82\x85\xe3\x81\x86\xe3\x81\x8b\xe3\x81\x84", true },
+    // "ま"
+    { "\xe3\x81\xbe", false },
+    // "まめ"
+    { "\xe3\x81\xbe\xe3\x82\x81", false },
+  };
+
+  for (size_t i = 0; i < arraysize(kTests2); ++i) {
+    entry.set_key(kTests2[i].entry_key);
+    EXPECT_EQ(kTests2[i].expect_result, predictor.LookupEntry(
+        // "し"
+        "\xe3\x81\x97", "", expanded.get(), &entry, NULL, &results))
+        << kTests2[i].entry_key;
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, GetMatchTypeFromInputRoman) {
+  // We have to define this here,
+  // because UserHistoryPredictor::MatchType is private
+  struct MatchTypeTestData {
+    const string target;
+    const UserHistoryPredictor::MatchType expect_type;
+  };
+
+  // Roman
+  // preedit: "あｋ"
+  // input_key: "あ"
+  // key_base: "あ"
+  // key_expanded: "か","き","く","け", "こ"
+  scoped_ptr<Trie<string> > expanded(new Trie<string>);
+  // "か", "か"
+  expanded->AddEntry("\xe3\x81\x8b", "\xe3\x81\x8b");
+  // "き", "き"
+  expanded->AddEntry("\xe3\x81\x8d", "\xe3\x81\x8d");
+  // "く", "く"
+  expanded->AddEntry("\xe3\x81\x8f", "\xe3\x81\x8f");
+  // "け", "け"
+  expanded->AddEntry("\xe3\x81\x91", "\xe3\x81\x91");
+  // "こ", "こ"
+  expanded->AddEntry("\xe3\x81\x93", "\xe3\x81\x93");
+
+  const MatchTypeTestData kTests1[] = {
+    { "", UserHistoryPredictor::NO_MATCH },
+    // "い"
+    { "\xe3\x81\x84", UserHistoryPredictor::NO_MATCH },
+    // "あ"
+    { "\xe3\x81\x82", UserHistoryPredictor::RIGHT_PREFIX_MATCH },
+    // "あい"
+    { "\xe3\x81\x82\xe3\x81\x84", UserHistoryPredictor::NO_MATCH },
+    // "あか"
+    { "\xe3\x81\x82\xe3\x81\x8b", UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "あかい"
+    { "\xe3\x81\x82\xe3\x81\x8b\xe3\x81\x84",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+  };
+
+  // ARRAYSIZE is less safe than arraysize, however we can't use
+  // arraysize for inner defined class.
+  // Please see base/port.h
+  for (size_t i = 0; i < ARRAYSIZE(kTests1); ++i) {
+    EXPECT_EQ(kTests1[i].expect_type,
+              UserHistoryPredictor::GetMatchTypeFromInput(
+                  // "あ", "あ"
+                  "\xe3\x81\x82", "\xe3\x81\x82",
+                  expanded.get(), kTests1[i].target))
+        << kTests1[i].target;
+  }
+
+  // only expanded
+  // preedit: "ｋ"
+  // input_key: ""
+  // key_base: ""
+  // key_expanded: "か","き","く","け", "こ"
+  const MatchTypeTestData kTests2[] = {
+    { "", UserHistoryPredictor::NO_MATCH },
+    // "い"
+    { "\xe3\x81\x84", UserHistoryPredictor::NO_MATCH },
+    // "いか"
+    { "\xe3\x81\x84\xe3\x81\x8b", UserHistoryPredictor::NO_MATCH },
+    // "か"
+    { "\xe3\x81\x8b", UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "かいがい"
+    { "\xe3\x81\x8b\xe3\x81\x84\xe3\x81\x8c\xe3\x81\x84",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+  };
+
+  for (size_t i = 0; i < ARRAYSIZE(kTests2); ++i) {
+    EXPECT_EQ(kTests2[i].expect_type,
+              UserHistoryPredictor::GetMatchTypeFromInput(
+                  "", "", expanded.get(), kTests2[i].target))
+        << kTests2[i].target;
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, GetMatchTypeFromInputKana) {
+  // We have to define this here,
+  // because UserHistoryPredictor::MatchType is private
+  struct MatchTypeTestData {
+    const string target;
+    const UserHistoryPredictor::MatchType expect_type;
+  };
+
+  // Kana
+  // preedit: "あし"
+  // input_key: "あし"
+  // key_base: "あ"
+  // key_expanded: "し","じ"
+  scoped_ptr<Trie<string> > expanded(new Trie<string>);
+  // "し", "し"
+  expanded->AddEntry("\xe3\x81\x97", "\xe3\x81\x97");
+  // "じ", "じ"
+  expanded->AddEntry("\xe3\x81\x98", "\xe3\x81\x98");
+
+  const MatchTypeTestData kTests1[] = {
+    { "", UserHistoryPredictor::NO_MATCH },
+    // "い"
+    { "\xe3\x81\x84", UserHistoryPredictor::NO_MATCH },
+    // "いし"
+    { "\xe3\x81\x84\xe3\x81\x97", UserHistoryPredictor::NO_MATCH },
+    // "あ"
+    { "\xe3\x81\x82", UserHistoryPredictor::RIGHT_PREFIX_MATCH },
+    // "あし"
+    { "\xe3\x81\x82\xe3\x81\x97", UserHistoryPredictor::EXACT_MATCH },
+    // "あじ"
+    { "\xe3\x81\x82\xe3\x81\x98",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "あした"
+    { "\xe3\x81\x82\xe3\x81\x97\xe3\x81\x9f",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "あじしお"
+    { "\xe3\x81\x82\xe3\x81\x98\xe3\x81\x97\xe3\x81\x8a",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+  };
+
+  // ARRAYSIZE is less safe than arraysize, however we can't use
+  // arraysize for inner defined class.
+  // Please see base/port.h
+  for (size_t i = 0; i < ARRAYSIZE(kTests1); ++i) {
+    EXPECT_EQ(kTests1[i].expect_type,
+              UserHistoryPredictor::GetMatchTypeFromInput(
+                  // "あし", "あ"
+                  "\xe3\x81\x82\xe3\x81\x97", "\xe3\x81\x82",
+                  expanded.get(), kTests1[i].target))
+        << kTests1[i].target;
+  }
+
+  // only expanded
+  // preedit: "し"
+  // input_key: "し"
+  // key_base: ""
+  // key_expanded: "し","じ"
+  const MatchTypeTestData kTests2[] = {
+    { "", UserHistoryPredictor::NO_MATCH },
+    // "い"
+    { "\xe3\x81\x84", UserHistoryPredictor::NO_MATCH },
+    // "し"
+    { "\xe3\x81\x97", UserHistoryPredictor::EXACT_MATCH },
+    // "じ"
+    { "\xe3\x81\x98", UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "しじみ"
+    { "\xe3\x81\x97\xe3\x81\x98\xe3\x81\xbf",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+    // "じかん"
+    { "\xe3\x81\x98\xe3\x81\x8b\xe3\x82\x93",
+      UserHistoryPredictor::LEFT_PREFIX_MATCH },
+  };
+
+  for (size_t i = 0; i < ARRAYSIZE(kTests2); ++i) {
+    EXPECT_EQ(kTests2[i].expect_type,
+              UserHistoryPredictor::GetMatchTypeFromInput(
+                  // "し"
+                  "\xe3\x81\x97", "", expanded.get(), kTests2[i].target))
+        << kTests2[i].target;
+  }
+}
+
+namespace {
+void InitSegmentsFromInputSequence(const string &text,
+                                   composer::Composer *composer,
+                                   Segments *segments) {
+  DCHECK(composer);
+  DCHECK(segments);
+  const char *begin = text.data();
+  const char *end = text.data() + text.size();
+  size_t mblen = 0;
+
+  while (begin < end) {
+    commands::KeyEvent key;
+    const char32 w = Util::UTF8ToUCS4(begin, end, &mblen);
+    if (Util::GetCharacterSet(w) == Util::ASCII) {
+      key.set_key_code(*begin);
+    } else {
+      key.set_key_code('?');
+      key.set_key_string(string(begin, mblen));
+    }
+    begin += mblen;
+    composer->InsertCharacterKeyEvent(key);
+  }
+
+  segments->set_request_type(Segments::PREDICTION);
+  segments->set_composer(composer);
+  Segment *segment = segments->add_segment();
+  CHECK(segment);
+  string query;
+  composer->GetQueryForPrediction(&query);
+  segment->set_key(query);
+}
+}  // namespace
+
+TEST_F(UserHistoryPredictorTest, GetInputKeyFromSegmentsRoman) {
+  scoped_ptr<composer::Table> table(new composer::Table);
+  scoped_ptr<composer::Composer> composer(new composer::Composer);
+  Segments segments;
+
+  table->LoadFromFile("system://romanji-hiragana.tsv");
+  composer->SetTableForUnittest(table.get());
+  InitSegmentsFromInputSequence("gu-g", composer.get(), &segments);
+
+  {
+    FLAGS_enable_expansion_for_user_history_predictor = true;
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "ぐーｇ"
+    EXPECT_EQ("\xe3\x81\x90\xe3\x83\xbc\xef\xbd\x87", input_key);
+    // "ぐー"
+    EXPECT_EQ("\xe3\x81\x90\xe3\x83\xbc", base);
+    EXPECT_TRUE(expanded.get() != NULL);
+    string value;
+    size_t key_length = 0;
+    bool has_subtrie = false;
+    EXPECT_TRUE(
+        // "ぐ"
+        expanded->LookUpPrefix("\xe3\x81\x90",
+                               &value, &key_length, &has_subtrie));
+    // "ぐ"
+    EXPECT_EQ("\xe3\x81\x90", value);
+  }
+
+  {
+    FLAGS_enable_expansion_for_user_history_predictor = false;
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "ぐー"
+    EXPECT_EQ("\xe3\x81\x90\xe3\x83\xbc", input_key);
+    // "ぐー"
+    EXPECT_EQ("\xe3\x81\x90\xe3\x83\xbc", base);
+    EXPECT_TRUE(expanded.get() == NULL);
+  }
+}
+
+namespace {
+uint32 GetRandomAscii() {
+  return static_cast<uint32>(' ') +
+      Util::Random(static_cast<uint32>('~' - ' '));
+}
+}  // namespace
+
+TEST_F(UserHistoryPredictorTest, GetInputKeyFromSegmentsRomanRandom) {
+  FLAGS_enable_expansion_for_user_history_predictor = true;
+  scoped_ptr<composer::Table> table(new composer::Table);
+  scoped_ptr<composer::Composer> composer(new composer::Composer);
+  Segments segments;
+
+  table->LoadFromFile("system://romanji-hiragana.tsv");
+  composer->SetTableForUnittest(table.get());
+
+  for (size_t i = 0; i < 1000; ++i) {
+    composer.reset(new composer::Composer);
+    composer->SetTableForUnittest(table.get());
+    const int len = 1 + Util::Random(4);
+    DCHECK_GE(len, 1);
+    DCHECK_LE(len, 5);
+    string input;
+    for (size_t j = 0; j < len; ++j) {
+      input += GetRandomAscii();
+    }
+    InitSegmentsFromInputSequence(input, composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+  }
+}
+
+// Found by random test.
+// input_key != base by compoesr modification.
+TEST_F(UserHistoryPredictorTest, GetInputKeyFromSegmentsShouldNotCrash) {
+  FLAGS_enable_expansion_for_user_history_predictor = true;
+  scoped_ptr<composer::Table> table(new composer::Table);
+  scoped_ptr<composer::Composer> composer(new composer::Composer);
+  Segments segments;
+
+  table->LoadFromFile("system://romanji-hiragana.tsv");
+  composer->SetTableForUnittest(table.get());
+  {
+    InitSegmentsFromInputSequence("8,+", composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+  }
+}
+
+TEST_F(UserHistoryPredictorTest, GetInputKeyFromSegmentsRomanN) {
+  FLAGS_enable_expansion_for_user_history_predictor = true;
+  scoped_ptr<composer::Table> table(new composer::Table);
+  scoped_ptr<composer::Composer> composer(new composer::Composer);
+  Segments segments;
+
+  table->LoadFromFile("system://romanji-hiragana.tsv");
+  composer->SetTableForUnittest(table.get());
+  {
+    InitSegmentsFromInputSequence("n", composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "ｎ"
+    EXPECT_EQ("\xef\xbd\x8e", input_key);
+    EXPECT_EQ("", base);
+    EXPECT_TRUE(expanded.get() != NULL);
+    string value;
+    size_t key_length = 0;
+    bool has_subtrie = false;
+    EXPECT_TRUE(
+        // "な"
+        expanded->LookUpPrefix("\xe3\x81\xaa",
+                               &value, &key_length, &has_subtrie));
+    // "な"
+    EXPECT_EQ("\xe3\x81\xaa", value);
+  }
+
+  composer.reset(new composer::Composer);
+  composer->SetTableForUnittest(table.get());
+  segments.Clear();
+  {
+    InitSegmentsFromInputSequence("nn", composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "ん"
+    EXPECT_EQ("\xe3\x82\x93", input_key);
+    // "ん"
+    EXPECT_EQ("\xe3\x82\x93", base);
+    EXPECT_TRUE(expanded.get() == NULL);
+  }
+
+  composer.reset(new composer::Composer);
+  composer->SetTableForUnittest(table.get());
+  segments.Clear();
+  {
+    InitSegmentsFromInputSequence("n'", composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "ん"
+    EXPECT_EQ("\xe3\x82\x93", input_key);
+    // "ん"
+    EXPECT_EQ("\xe3\x82\x93", base);
+    EXPECT_TRUE(expanded.get() == NULL);
+  }
+
+  composer.reset(new composer::Composer);
+  composer->SetTableForUnittest(table.get());
+  segments.Clear();
+  {
+    InitSegmentsFromInputSequence("n'n", composer.get(), &segments);
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "んｎ"
+    EXPECT_EQ("\xe3\x82\x93\xef\xbd\x8e", input_key);
+    // "ん"
+    EXPECT_EQ("\xe3\x82\x93", base);
+    EXPECT_TRUE(expanded.get() != NULL);
+    string value;
+    size_t key_length = 0;
+    bool has_subtrie = false;
+    EXPECT_TRUE(
+        // "な"
+        expanded->LookUpPrefix("\xe3\x81\xaa",
+                               &value, &key_length, &has_subtrie));
+    // "な"
+    EXPECT_EQ("\xe3\x81\xaa", value);
+  }
+}
+
+
+TEST_F(UserHistoryPredictorTest, GetInputKeyFromSegmentsKana) {
+  scoped_ptr<composer::Table> table(new composer::Table);
+  scoped_ptr<composer::Composer> composer(new composer::Composer);
+  Segments segments;
+
+  table->LoadFromFile("system://kana.tsv");
+  composer->SetTableForUnittest(table.get());
+  // "あか"
+  InitSegmentsFromInputSequence("\xe3\x81\x82\xe3\x81\x8b",
+                                composer.get(), &segments);
+
+  {
+    FLAGS_enable_expansion_for_user_history_predictor = true;
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "あか"
+    EXPECT_EQ("\xe3\x81\x82\xe3\x81\x8b", input_key);
+    // "あ"
+    EXPECT_EQ("\xe3\x81\x82", base);
+    EXPECT_TRUE(expanded.get() != NULL);
+    string value;
+    size_t key_length = 0;
+    bool has_subtrie = false;
+    EXPECT_TRUE(
+        // "が"
+        expanded->LookUpPrefix("\xe3\x81\x8c",
+                               &value, &key_length, &has_subtrie));
+    // "が"
+    EXPECT_EQ("\xe3\x81\x8c", value);
+  }
+
+  {
+    FLAGS_enable_expansion_for_user_history_predictor = false;
+    string input_key;
+    string base;
+    scoped_ptr<Trie<string> > expanded;
+    UserHistoryPredictor::GetInputKeyFromSegments(segments,
+                                                  &input_key,
+                                                  &base,
+                                                  &expanded);
+    // "あか"
+    EXPECT_EQ("\xe3\x81\x82\xe3\x81\x8b", input_key);
+    // "あか"
+    EXPECT_EQ("\xe3\x81\x82\xe3\x81\x8b", base);
+    EXPECT_TRUE(expanded.get() == NULL);
+  }
+}
 }  // namespace mozc
